#!/usr/bin/env python3
"""
SneakerPicks Feed Sync v3 — Category filtering + Size dedup + Cross-shop matching + Price history
Handles Webgains feeds with:
- Sneaker-only filtering (category + name-based)
- Size variant aggregation (one offer per product-per-shop)
- Cross-shop product matching (EAN/fingerprint)
- Price history logging
"""
import requests
import psycopg2
import psycopg2.extras
import re
import hashlib
import time
import json
import sys
from collections import defaultdict

# ── Config ──
WEBGAINS_API = "https://platform-api.webgains.com"
PUBLISHER_ID = "1347930"
WEBGAINS_TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"

DB_URL = "postgresql://supabase_admin:HzWrn2rMMxUYWO6OT5X0PScIVEvSjQMup9YSLIrkPQ@172.18.0.5:5432/sneakerpicks"

CAMPAIGNS = {
    "manify.nl": "1622230",
    "ladify.nl": "1622250",
    "thehike.nl": "1622240",
    "mellowed.nl": "1622260",
}

HEADERS = {
    "Authorization": f"Bearer {WEBGAINS_TOKEN}",
    "Content-Type": "application/json",
    "Accept": "application/json",
}

PRODUCT_FIELDS = [
    "title", "image_link", "link", "price", "brand", "description", "id",
    "sale_price", "gtin", "mpn", "colour", "size", "availability", "condition",
    "product_type", "google_product_category"
]

# ── Sneaker Detection ──

# Known sneaker brands (always accept footwear from these)
SNEAKER_BRANDS = {
    'nike', 'adidas', 'adidas originals', 'new balance', 'puma', 'asics', 'asisc',
    'jordan', 'reebok', 'converse', 'vans', 'saucony', 'hoka', 'on running', 'on',
    'salomon', 'mizuno', 'diadora', 'karhu', 'filling pieces', 'axel arigato',
    'common projects', 'autry', 'golden goose', 'versace', 'gucci', 'balenciaga',
    'alexander mcqueen', 'off-white', 'fear of god', 'rick owens', 'maison margiela',
    'mm6 maison margiela', 'y-3', 'stone island', 'moncler', 'premiata', 'ghoud',
    'philippe model', 'p448', 'date', 'd.a.t.e.', 'womsh', 'copenhagen', 
    'copenhagen studios', 'nubikk', 'mason garments', 'mercer amsterdam',
    'floris van bommel', 'cruyff', 'android homme', 'etq amsterdam', 'ekn',
    'yeezy', 'li-ning', 'anta', 'hi-tec', 'hts', 'mikakus',
    'cph', 'cph40', 'birkenstock', 'clarks', 'dr. martens',
    'lacoste', 'tommy hilfiger', 'calvin klein', 'hugo boss', 'boss',
    'fila', 'k-swiss', 'le coq sportif', 'ellesse', 'umbro', 'lotto',
    'under armour', 'brooks', 'altra', 'topo athletic',
}

# Category patterns that indicate sneakers/footwear
SNEAKER_CATEGORY_PATTERNS = [
    'sneaker', 'trainer', 'running shoe', 'sportschoenen', 'footwear',
    'athletic shoe', 'casual shoe', 'schoen', 'shoes', 'boots',  # boots ok for sneaker brands
    'apparel & accessories > shoes',
]

# Category patterns that indicate NOT sneakers
NON_SNEAKER_CATEGORY_PATTERNS = [
    'clothing', 'apparel > clothing', 'accessori', 'bag', 'hat', 'cap',
    'fragrance', 'home', 'art', 'book', 'candle', 'poster',
    'shirt', 'jacket', 'trouser', 'pant', 'hoodie', 'sweater',
]

# Title exclusion patterns (definitely not sneakers)
EXCLUDE_TITLE_PATTERNS = [
    r'\bt-shirt', r'\btshirt', r'\btee\b', r'\bhoodie', r'\bhoody\b',
    r'\bsweater', r'\bsweatshirt', r'\bjacket\b', r'\bcoat\b',
    r'\bpants\b', r'\btrouser', r'\bjogger', r'\bshorts\b', r'\bjeans\b',
    r'\bshirt\b', r'\bpolo\b', r'\bvest\b', r'\bcap\b', r'\bhat\b',
    r'\bbeanie\b', r'\bscarf\b', r'\bglove', r'\bsock\b', r'\bsocks\b',
    r'\bbackpack', r'\bbag\b', r'\bwallet\b', r'\bbelt\b', r'\bwatch\b',
    r'\bsunglasses', r'\bunderwear', r'\bboxer', r'\bswim',
    r'\bperfume', r'\bfragrance', r'\bdeodorant', r'\bcandle',
    r'\bkeychain', r'\bkeyring', r'\bphone case', r'\bairpod',
    r'\binsole', r'\blaces\b', r'\bcleaner\b', r'\bprotector\b',
    r'\bspray\b', r'\bbrush\b', r'\btracksuit', r'\bwindbreaker',
    r'\bpuffer\b', r'\bfleece\b', r'\bcardigan', r'\bcrewneck',
    r'\blongsleeve', r'\btank top', r'\blegging', r'\bskirt\b',
    r'\bdress\b', r'\bjumpsuit', r'\bblanket', r'\bpillow',
    r'\bmug\b', r'\bcup\b', r'\bbottle\b', r'\bincense',
    r'\bsticker\b', r'\bpatch\b', r'\blanyard', r'\btote\b',
    r'\bflip.?flop', r'\bsandal', r'\bslipper', r'\bclog\b',
    r'\bloafer', r'\bpump\b', r'\bheel\b', r'\bwedge\b',
    r'\bdress shoe', r'\boxford\b', r'\bderby\b', r'\bbrogue\b',
    r'\bchelsea boot',  # debatable but not a sneaker
]

# Known sneaker model patterns (accept even without category)
SNEAKER_MODEL_PATTERNS = [
    r'air max', r'air force', r'air jordan', r'\baj\d', r'\bdunk\b',
    r'\bwaffle\b', r'\bcortez\b', r'\bblaze\b', r'\bpegasus\b',
    r'\b574\b', r'\b990\b', r'\b992\b', r'\b993\b', r'\b550\b', r'\b530\b', r'\b2002r?\b',
    r'\bgel.?lyte', r'\bgel.?kayano', r'\bgel.?1130', r'\bgel.?nyc',
    r'\bsamba\b', r'\bgazelle\b', r'\bsuperstar\b', r'\bstan smith\b', r'\bforum\b',
    r'\bozweego\b', r'\byung\b', r'\bnite jogger', r'\bultraboost\b', r'\bnmd\b',
    r'\bold skool\b', r'\bsk8.?hi\b', r'\bera\b', r'\bauthentic\b', r'\bslip.?on\b',
    r'\bchuck taylor\b', r'\ball star\b', r'\bone star\b', r'\bjack purcell\b',
    r'\bclassic leather\b', r'\bclub c\b', r'\bquestion\b',
    r'\bsuede\b', r'\brs-?x\b', r'\brs.?connect', r'\beasy rider\b',
    r'\bshadow\b.*\b6000\b', r'\bjazz\b', r'\bgrid\b',
    r'\bclifton\b', r'\bbondi\b', r'\bmafate\b', r'\bspeedgoat\b',
    r'\bxt-?\d', r'\bspeedcross\b', r'\bwave rider\b', r'\bmorelia\b',
    r'\bchunky\b', r'\bretro\b', r'\bplatform\b',
    r'\bprogrind\b', r'\bprogrid\b',
    r'\bcloud\b.*\bmonster', r'\bcloudmonster\b',
]


def is_sneaker(title, brand, product_type, google_category):
    """Determine if a product is a sneaker. Returns (bool, reason)."""
    title_lower = (title or '').lower()
    brand_lower = (brand or '').lower()
    ptype_lower = (product_type or '').lower()
    gcat_lower = (google_category or '').lower()
    
    # Step 1: Exclude by title (clothing, accessories, etc.)
    for pat in EXCLUDE_TITLE_PATTERNS:
        if re.search(pat, title_lower):
            return False, f'title_exclude:{pat}'
    
    # Step 2: Check category exclusions
    for pat in NON_SNEAKER_CATEGORY_PATTERNS:
        if pat in ptype_lower or pat in gcat_lower:
            return False, f'category_exclude:{pat}'
    
    # Step 3: Check category inclusions
    for pat in SNEAKER_CATEGORY_PATTERNS:
        if pat in ptype_lower or pat in gcat_lower:
            return True, f'category_match:{pat}'
    
    # Step 4: Known sneaker brand → accept (footwear from sneaker brands)
    if brand_lower in SNEAKER_BRANDS:
        # But reject if it's clearly not footwear
        return True, f'brand_match:{brand_lower}'
    
    # Step 5: Known sneaker model patterns in name
    for pat in SNEAKER_MODEL_PATTERNS:
        if re.search(pat, title_lower):
            return True, f'model_match:{pat}'
    
    # Step 6: No signal → reject
    return False, 'no_match'


# ── Size Normalization ──

SIZE_PATTERNS = [
    # "EU 42", "EU42", "Maat 42"
    r'\b(?:eu|eur|maat)\s*(\d{2}(?:[.,]\d)?)\b',
    # "42.5", "43" at end of string or followed by non-alpha
    r'\b(\d{2}(?:[.,]\d{1,2})?)\s*$',
    # "Size 42"
    r'\bsize\s*(\d{2}(?:[.,]\d)?)\b',
    # US sizes
    r'\bus\s*(\d{1,2}(?:[.,]\d)?)\b',
]

def extract_size(title, size_field):
    """Extract size from size field or title."""
    if size_field and size_field.strip():
        return size_field.strip()
    
    for pat in SIZE_PATTERNS:
        m = re.search(pat, title.lower())
        if m:
            return f"EU {m.group(1)}"
    return None


def normalize_product_name(title, brand):
    """Remove size info and normalize product name for grouping."""
    name = title.strip()
    # Remove size suffixes like "42", "EU 42", "Maat 42.5"
    name = re.sub(r'\s+(?:eu|eur|maat|size)\s*\d{2}(?:[.,]\d{1,2})?\s*$', '', name, flags=re.I)
    name = re.sub(r'\s+\d{2}(?:[.,]\d{1,2})?\s*$', '', name)
    # Remove trailing whitespace/dashes
    name = name.strip(' -–—')
    return name


def make_fingerprint(brand, name, colorway):
    """Create a match fingerprint for cross-shop dedup."""
    parts = [
        (brand or '').lower().strip(),
        re.sub(r'[^\w\s]', '', (name or '').lower()).strip(),
        (colorway or '').lower().strip(),
    ]
    raw = '|'.join(parts)
    return hashlib.md5(raw.encode()).hexdigest()


# ── Utilities ──

def slugify(text):
    text = text.lower().strip()
    text = re.sub(r'[^\w\s-]', '', text)
    text = re.sub(r'[\s_]+', '-', text)
    text = re.sub(r'-+', '-', text)
    return text[:200]


def parse_price(val):
    if not val:
        return None
    val = str(val).replace(',', '.').strip()
    match = re.search(r'[\d.]+', val)
    if match:
        try:
            return float(match.group())
        except:
            return None
    return None


# ── Webgains API ──

def get_feeds():
    """Get all unique feeds across campaigns."""
    all_feeds = {}
    for site, camp_id in CAMPAIGNS.items():
        url = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{camp_id}/feeds"
        try:
            r = requests.get(url, headers=HEADERS, params={"size": 100}, timeout=30)
            r.raise_for_status()
            data = r.json()
            for f in data.get("data", []):
                fid = f["id"]
                if fid not in all_feeds:
                    all_feeds[fid] = {
                        "id": fid,
                        "name": f.get("name", ""),
                        "program_name": f.get("program", {}).get("name", ""),
                        "program_id": str(f.get("program", {}).get("id", "")),
                        "product_count": f.get("products_successfully_stored", 0),
                        "campaign_id": camp_id,
                    }
            print(f"  {site} ({camp_id}): {len(data.get('data', []))} feeds")
        except Exception as e:
            print(f"  {site} ERROR: {e}")
    return all_feeds


def fetch_all_products_cursor(campaign_id, feed_id, max_pages=200):
    """Fetch ALL products using cursor-based pagination."""
    url = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{campaign_id}/feeds/products"
    all_products = []
    seen_ids = set()
    cursor = None

    for page in range(max_pages):
        params = {
            "feeds[]": [feed_id],
            "size": 1000,
            "fields[]": PRODUCT_FIELDS,
        }
        if cursor:
            params["search_cursor"] = cursor

        try:
            r = requests.get(url, headers=HEADERS, params=params, timeout=60)
            r.raise_for_status()
            data = r.json()
        except Exception as e:
            print(f"    Page {page} error: {e}")
            break

        products = data.get("data", [])
        if not products:
            break

        pagination = data.get("pagination", {})
        new_cursor = pagination.get("search_cursor")

        new_products = []
        for p in products:
            pid = p.get("id")
            if pid and pid not in seen_ids:
                seen_ids.add(pid)
                new_products.append(p)

        if not new_products:
            break

        all_products.extend(new_products)

        if page % 10 == 0 or len(new_products) < 900:
            print(f"    Page {page}: +{len(new_products)} (total: {len(all_products)})")

        if new_cursor == cursor or not new_cursor:
            break

        cursor = new_cursor
        time.sleep(0.3)

    return all_products


# ── Import Pipeline ──

def process_feed(cur, conn, feed_info, provider_id, brand_cache, ean_product_cache, fingerprint_cache, only_feed=None):
    """Process a single feed: fetch → filter → dedupe sizes → upsert."""
    feed_ext_id = feed_info["id"]
    fname = feed_info["name"]
    camp_id = feed_info["campaign_id"]

    if only_feed and str(feed_ext_id) != str(only_feed):
        return 0, 0, 0

    print(f"\n{'='*60}")
    print(f"Feed: {fname} (ext_id={feed_ext_id})")
    print(f"{'='*60}")

    # Upsert feed in DB
    cur.execute("""
        INSERT INTO feeds (provider_id, external_id, name, merchant_name, product_count)
        VALUES (%s, %s, %s, %s, %s)
        ON CONFLICT (provider_id, external_id) DO UPDATE SET
            name = EXCLUDED.name, merchant_name = EXCLUDED.merchant_name
        RETURNING id
    """, (provider_id, str(feed_ext_id), fname, feed_info["program_name"], feed_info.get("product_count") or 0))
    db_feed_id = cur.fetchone()["id"]
    conn.commit()

    # Fetch products
    products = fetch_all_products_cursor(camp_id, feed_ext_id)
    if not products:
        print("  No products, skipping")
        return 0, 0, 0

    print(f"  Fetched: {len(products)} raw products")

    # ── Step 1: Filter for sneakers ──
    accepted = []
    rejected_count = 0
    reject_batch = []

    for p in products:
        title = (p.get("title") or "").strip()
        brand_name = (p.get("brand") or "").strip()
        ptype_raw = p.get("product_type") or ""
        if isinstance(ptype_raw, list):
            ptype_raw = ptype_raw[0] if ptype_raw else ""
        ptype = str(ptype_raw).strip()
        gcat_raw = p.get("google_product_category") or ""
        if isinstance(gcat_raw, list):
            gcat_raw = gcat_raw[0] if gcat_raw else ""
        gcat = str(gcat_raw).strip()

        is_snkr, reason = is_sneaker(title, brand_name, ptype, gcat)

        if is_snkr:
            accepted.append(p)
        else:
            rejected_count += 1
            reject_batch.append((db_feed_id, str(p.get("id", "")), title, brand_name, ptype or gcat, reason))

    # Log rejects in batches
    if reject_batch:
        psycopg2.extras.execute_batch(cur, """
            INSERT INTO import_rejects (feed_id, external_id, name, brand, category, reason)
            VALUES (%s, %s, %s, %s, %s, %s)
        """, reject_batch, page_size=500)
        conn.commit()

    print(f"  Filtered: {len(accepted)} accepted, {rejected_count} rejected")

    # ── Step 2: Group by product (size dedup) ──
    # Group key: brand_lower + normalized_name
    groups = defaultdict(list)
    for p in accepted:
        title = (p.get("title") or "").strip()
        brand_name = (p.get("brand") or "").strip()
        norm_name = normalize_product_name(title, brand_name)
        key = f"{brand_name.lower()}||{norm_name.lower()}"
        groups[key].append(p)

    print(f"  Grouped into {len(groups)} unique products (from {len(accepted)} variants)")

    # ── Step 3: Upsert products + offers ──
    new_products = 0
    total_offers = 0
    price_history_batch = []

    for group_key, variants in groups.items():
        # Pick the "best" variant for product info (first one with most data)
        primary = variants[0]
        title = normalize_product_name((primary.get("title") or "").strip(), (primary.get("brand") or "").strip())
        brand_name = (primary.get("brand") or "").strip()
        image = (primary.get("image_link") or "").strip()
        colour = (primary.get("colour") or "").strip()
        description = (primary.get("description") or "")[:500]
        ptype_raw2 = primary.get("product_type") or ""
        if isinstance(ptype_raw2, list):
            ptype_raw2 = ptype_raw2[0] if ptype_raw2 else ""
        ptype = str(ptype_raw2).strip()
        gcat_raw2 = primary.get("google_product_category") or ""
        if isinstance(gcat_raw2, list):
            gcat_raw2 = gcat_raw2[0] if gcat_raw2 else ""
        gcat = str(gcat_raw2).strip()

        # Collect all EANs and sizes
        sizes_data = []
        all_eans = set()
        min_price = None
        min_original_price = None
        best_link = (primary.get("link") or "").strip()

        for v in variants:
            price = parse_price(v.get("sale_price") or v.get("price"))
            orig_price = parse_price(v.get("price")) if v.get("sale_price") else None
            link = (v.get("link") or "").strip()

            gtin_val = v.get("gtin") or ""
            if isinstance(gtin_val, list):
                gtin_val = gtin_val[0] if gtin_val else ""
            ean = str(gtin_val).strip() or None

            size_val = extract_size((v.get("title") or ""), (v.get("size") or ""))

            if ean:
                all_eans.add(ean)

            if price:
                size_entry = {"size": size_val or "One Size", "price": price}
                if ean:
                    size_entry["ean"] = ean
                if link:
                    size_entry["url"] = link
                sizes_data.append(size_entry)

                if min_price is None or price < min_price:
                    min_price = price
                    best_link = link
                if orig_price and (min_original_price is None or orig_price < min_original_price):
                    min_original_price = orig_price

        if not min_price or not title or not best_link:
            continue

        primary_ean = next(iter(all_eans), None)

        # ── Cross-shop matching: find or create product ──
        product_id = None

        # Try EAN match first
        for ean in all_eans:
            if ean in ean_product_cache:
                product_id = ean_product_cache[ean]
                break
            cur.execute("SELECT id FROM products WHERE ean = %s", (ean,))
            row = cur.fetchone()
            if row:
                product_id = row["id"]
                ean_product_cache[ean] = product_id
                break

        # Try fingerprint match
        if not product_id:
            fp = make_fingerprint(brand_name, title, colour)
            if fp in fingerprint_cache:
                product_id = fingerprint_cache[fp]
            else:
                cur.execute("SELECT id FROM products WHERE match_fingerprint = %s", (fp,))
                row = cur.fetchone()
                if row:
                    product_id = row["id"]
                    fingerprint_cache[fp] = product_id

        # Create new product
        if not product_id:
            brand_slug = slugify(brand_name) if brand_name else "unknown"
            if brand_slug not in brand_cache:
                try:
                    cur.execute("""
                        INSERT INTO brands (name, slug) VALUES (%s, %s)
                        ON CONFLICT (slug) DO UPDATE SET name = EXCLUDED.name
                        RETURNING id
                    """, (brand_name or "Unknown", brand_slug))
                    brand_cache[brand_slug] = cur.fetchone()["id"]
                except Exception:
                    conn.rollback()
                    cur.execute("SELECT id FROM brands WHERE slug = %s", (brand_slug,))
                    row = cur.fetchone()
                    if row:
                        brand_cache[brand_slug] = row["id"]
            brand_id = brand_cache.get(brand_slug)

            fp = make_fingerprint(brand_name, title, colour)
            base_slug = slugify(f"{brand_name}-{title}")[:180]
            slug_hash = hashlib.md5(f"{feed_ext_id}-{group_key}".encode()).hexdigest()[:8]
            product_slug = f"{base_slug}-{slug_hash}"

            try:
                cur.execute("""
                    INSERT INTO products (name, slug, brand_id, brand, image_url, description, ean,
                                          colorway, brand_normalized, model_normalized, colorway_normalized,
                                          match_fingerprint, category, product_type, first_seen_at)
                    VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, NOW())
                    RETURNING id
                """, (title, product_slug, brand_id, brand_name, image, description,
                      primary_ean, colour,
                      brand_name.lower() if brand_name else None,
                      title.lower() if title else None,
                      colour.lower() if colour else None,
                      fp, gcat or ptype, ptype))
                product_id = cur.fetchone()["id"]
                new_products += 1

                # Cache
                fingerprint_cache[fp] = product_id
                for ean in all_eans:
                    ean_product_cache[ean] = product_id
            except psycopg2.errors.UniqueViolation:
                conn.rollback()
                cur.execute("SELECT id FROM products WHERE slug = %s", (product_slug,))
                row = cur.fetchone()
                if row:
                    product_id = row["id"]
                else:
                    continue
            except Exception as e:
                conn.rollback()
                print(f"    Error creating product '{title}': {e}")
                continue

        # ── Upsert offer with aggregated sizes ──
        # Use first external_id as the canonical one for the offer
        ext_id = str(variants[0].get("id", "")).strip()
        sizes_json = json.dumps(sizes_data, ensure_ascii=False)

        try:
            cur.execute("""
                INSERT INTO product_offers (product_id, feed_id, external_id, price, original_price,
                                             currency, product_url, image_url, in_stock, sizes, last_seen_at)
                VALUES (%s, %s, %s, %s, %s, 'EUR', %s, %s, true, %s::jsonb, NOW())
                ON CONFLICT (feed_id, external_id) DO UPDATE SET
                    price = EXCLUDED.price,
                    original_price = EXCLUDED.original_price,
                    product_url = EXCLUDED.product_url,
                    image_url = EXCLUDED.image_url,
                    in_stock = true,
                    sizes = EXCLUDED.sizes,
                    last_seen_at = NOW(),
                    updated_at = NOW()
            """, (product_id, db_feed_id, ext_id, min_price, min_original_price,
                  best_link, image, sizes_json))
            total_offers += 1

            # Price history
            price_history_batch.append((product_id, db_feed_id, min_price))
        except Exception as e:
            conn.rollback()
            print(f"    Error upserting offer for '{title}': {e}")
            continue

        # Commit every 200 products
        if total_offers % 200 == 0:
            conn.commit()

    conn.commit()

    # ── Price history (only insert if price changed) ──
    for product_id, feed_id, price in price_history_batch:
        try:
            cur.execute("""
                INSERT INTO price_history (product_id, feed_id, price, recorded_at)
                SELECT %s, %s, %s, NOW()
                WHERE NOT EXISTS (
                    SELECT 1 FROM price_history
                    WHERE product_id = %s AND feed_id = %s AND price = %s
                    AND recorded_at > NOW() - INTERVAL '23 hours'
                )
            """, (product_id, feed_id, price, product_id, feed_id, price))
        except Exception:
            conn.rollback()
    conn.commit()

    # Update feed stats
    cur.execute("""
        UPDATE feeds SET product_count = %s, last_sync_at = NOW(), last_sync_status = 'success'
        WHERE id = %s
    """, (total_offers, db_feed_id))
    conn.commit()

    print(f"  ✓ {new_products} new products, {total_offers} offers, {len(price_history_batch)} price records")
    return new_products, total_offers, rejected_count


def main():
    only_feed = None
    clean_first = False

    for arg in sys.argv[1:]:
        if arg == '--clean':
            clean_first = True
        elif arg.startswith('--feed='):
            only_feed = arg.split('=')[1]

    print("=== SneakerPicks Feed Sync v3 ===")
    print(f"  Options: only_feed={only_feed}, clean={clean_first}")
    print()

    conn = psycopg2.connect(DB_URL)
    conn.autocommit = False
    cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

    if clean_first:
        print("Cleaning old data...")
        cur.execute("""
            TRUNCATE product_offers, price_history, featured_products, collection_products,
                     collection_overrides, product_categories, product_tags, price_alerts,
                     import_rejects
            RESTART IDENTITY CASCADE
        """)
        cur.execute("DELETE FROM products")
        cur.execute("DELETE FROM feeds")
        cur.execute("DELETE FROM sync_logs")
        cur.execute("DELETE FROM brands WHERE slug != 'unknown'")
        conn.commit()
        print("Done.\n")

    # Get provider
    cur.execute("SELECT id FROM providers WHERE slug = 'webgains'")
    provider_id = cur.fetchone()["id"]

    # Caches
    brand_cache = {}
    cur.execute("SELECT id, slug FROM brands")
    for row in cur.fetchall():
        brand_cache[row["slug"]] = row["id"]

    ean_product_cache = {}
    fingerprint_cache = {}

    # If not clean, pre-load caches
    if not clean_first:
        cur.execute("SELECT id, ean, match_fingerprint FROM products WHERE ean IS NOT NULL OR match_fingerprint IS NOT NULL")
        for row in cur.fetchall():
            if row["ean"]:
                ean_product_cache[row["ean"]] = row["id"]
            if row["match_fingerprint"]:
                fingerprint_cache[row["match_fingerprint"]] = row["id"]

    print("Fetching feeds from Webgains API...")
    feeds = get_feeds()
    print(f"Found {len(feeds)} unique feeds\n")

    total_new = 0
    total_offers = 0
    total_rejected = 0

    for feed_ext_id, feed_info in sorted(feeds.items()):
        new, offers, rejected = process_feed(
            cur, conn, feed_info, provider_id,
            brand_cache, ean_product_cache, fingerprint_cache,
            only_feed=only_feed
        )
        total_new += new
        total_offers += offers
        total_rejected += rejected

    # ── Summary ──
    print(f"\n{'='*60}")
    print(f"SYNC COMPLETE")
    print(f"{'='*60}")
    print(f"New products: {total_new}")
    print(f"Total offers: {total_offers}")
    print(f"Rejected: {total_rejected}")

    cur.execute("SELECT COUNT(*) FROM products")
    print(f"Products in DB: {cur.fetchone()[0]}")
    cur.execute("SELECT COUNT(*) FROM product_offers")
    print(f"Offers in DB: {cur.fetchone()[0]}")
    cur.execute("SELECT COUNT(*) FROM import_rejects")
    print(f"Rejects logged: {cur.fetchone()[0]}")

    cur.execute("""
        SELECT f.name, COUNT(po.id) as offers, COUNT(DISTINCT po.product_id) as products
        FROM feeds f LEFT JOIN product_offers po ON po.feed_id = f.id
        GROUP BY f.name ORDER BY offers DESC
    """)
    print("\nPer feed:")
    for row in cur.fetchall():
        print(f"  {row['name']}: {row['offers']} offers, {row['products']} products")

    cur.execute("SELECT brand, COUNT(*) c FROM products GROUP BY brand ORDER BY c DESC LIMIT 15")
    print("\nTop brands:")
    for row in cur.fetchall():
        print(f"  {row[0] or '(empty)'}: {row[1]}")

    cur.close()
    conn.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""Sample products from Webgains feeds to check for sneakers."""
import requests
import json

WEBGAINS_API = "https://platform-api.webgains.com"
PUBLISHER_ID = "1347930"
TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
HEADERS = {"Authorization": f"Bearer {TOKEN}", "Accept": "application/json"}
CAMPAIGN_ID = "1622230"

FEEDS_TO_CHECK = [
    (17108, "Van Arendonk NL new"),
    (11218, "Van Arendonk NL"),
    (25538, "SikSilk NL"),
    (34710, "Sneakermood German Feed"),
]

SHOE_WORDS = ['sneaker', 'shoe', 'trainer', 'runner', 'boot', 'slide', 'sandal',
              'slipper', 'dunk', 'force 1', 'air max', 'jordan', 'yeezy',
              'ultraboost', 'superstar', 'old skool', 'gel-', '990', '550',
              'speedcat', 'spezial', 'schoen']

for feed_id, feed_name in FEEDS_TO_CHECK:
    print(f"\n{'='*60}")
    print(f"Feed: {feed_name} ({feed_id})")
    print(f"{'='*60}")
    
    url = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{CAMPAIGN_ID}/feeds/products"
    params = {"feedIds[]": [feed_id], "size": 100}
    
    try:
        r = requests.get(url, headers=HEADERS, params=params, timeout=30)
        print(f"Status: {r.status_code}")
        if r.status_code != 200:
            print(f"Error: {r.text[:300]}")
            continue
            
        data = r.json()
        products = data.get("data", [])
        total = data.get("pagination", {}).get("total", "?")
        print(f"Products in sample: {len(products)}, Total: {total}")
        
        if not products:
            continue
        
        # Count categories and check for sneakers
        cats = {}
        sneaker_count = 0
        for p in products:
            cat = p.get('product_type') or ['Unknown']
            if isinstance(cat, list):
                cat = cat[0] if cat else 'Unknown'
            cat = str(cat).strip()[:40]
            cats[cat] = cats.get(cat, 0) + 1
            
            title = (p.get('title') or '').lower()
            if any(w in title for w in SHOE_WORDS):
                sneaker_count += 1
        
        print(f"Categories: {sorted(cats.items(), key=lambda x: -x[1])[:10]}")
        print(f"Sneaker-matching titles: {sneaker_count}/{len(products)}")
        
        # Show sample products
        print(f"\nSample products:")
        for p in products[:15]:
            title = p.get('title', '?')[:55]
            brand = p.get('brand', '?')[:15]
            price = p.get('sale_price') or p.get('price') or '?'
            cat = p.get('product_type') or '?'
            if isinstance(cat, list):
                cat = cat[0] if cat else '?'
            print(f"  {brand:15s} | {cat!s:20s} | {title}")
    except Exception as e:
        print(f"Error: {e}")



import requests
import sys
sys.path.insert(0, r'C:\Users\Roel\clawd\scripts\demandgen-pipeline')
from config import WEBGAINS_API, PUBLISHER_ID, webgains_headers

# Check feed details for download URLs
cid = 1622230
url = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{cid}/feeds'
r = requests.get(url, headers=webgains_headers(), params={'size': 100})
d = r.json()
for f in d.get('data', []):
    count = f.get('products_successfully_stored', 0) or 0
    if count > 0:
        # Print all fields to find download URL
        name = f.get('name', '')
        fid = f['id']
        print(f'\nFeed {fid}: {name} ({count} products)')
        # Check for downloadable URL fields
        for key in sorted(f.keys()):
            if key not in ['program', 'name', 'id', 'products_successfully_stored']:
                val = f[key]
                if val and str(val).strip():
                    print(f'  {key}: {str(val)[:200]}')
        if 'program' in f and f['program']:
            for key in sorted(f['program'].keys()):
                val = f['program'][key]
                if val and str(val).strip():
                    print(f'  program.{key}: {str(val)[:200]}')
        break  # Just check first one for structure

# Also try Awin API
print('\n\n=== AWIN API ===')
awin_token = 'bebd7839-013a-434d-a3da-7db2ecefb6b1'
awin_headers = {'Authorization': f'Bearer {awin_token}'}
# List publishers/programmes
r2 = requests.get('https://api.awin.com/publishers', headers=awin_headers)
print(f'Awin publishers: {r2.status_code}')
if r2.ok:
    print(r2.text[:500])

# Try product search
r3 = requests.get('https://productdata.awin.com/datafeed/list/apikey/bebd7839-013a-434d-a3da-7db2ecefb6b1', timeout=30)
print(f'\nAwin product feeds list: {r3.status_code}')
if r3.ok:
    print(r3.text[:1000])



#!/usr/bin/env python3
"""Test pagination workarounds for feedIds[]."""
import requests

TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
PUBLISHER_ID = "1347930"
H = {"Authorization": f"Bearer {TOKEN}", "Accept": "application/json"}
url = f"https://platform-api.webgains.com/auth/publishers/{PUBLISHER_ID}/campaigns/1622230/feeds/products"

# Try page param
print("=== Try page param ===")
for page in [0, 1, 2]:
    params = {"feedIds[]": [18588], "size": 5, "page": page}
    r = requests.get(url, headers=H, params=params, timeout=10)
    data = r.json()
    prods = data.get("data", [])
    ids = [p.get("id") for p in prods]
    print(f"  page={page}: ids={ids[:3]}")

# Try sort + offset
print("\n=== Try sort ===")
for sort in ["id", "title", "price", "createdAt"]:
    params = {"feedIds[]": [18588], "size": 3, "sort": sort, "order": "asc"}
    r = requests.get(url, headers=H, params=params, timeout=10)
    data = r.json()
    prods = data.get("data", [])
    if prods:
        print(f"  sort={sort}: {prods[0].get('id')} | {prods[0].get('title','')[:40]}")

# Try max size
print("\n=== Try large size ===")
for sz in [2000, 5000, 10000]:
    params = {"feedIds[]": [18588], "size": sz}
    r = requests.get(url, headers=H, params=params, timeout=60)
    data = r.json()
    prods = data.get("data", [])
    unique_ids = len(set(p.get("id") for p in prods))
    print(f"  size={sz}: got {len(prods)} products, {unique_ids} unique IDs")
    if prods:
        # Check if all from baskets
        links = [p.get("link","") for p in prods[-3:]]
        baskets_count = sum(1 for l in links if "baskets" in l.lower() or "299740" in l)
        print(f"    Last 3 links are baskets: {baskets_count}/3")

# Try cursor/after param
print("\n=== Try cursor ===")
params = {"feedIds[]": [18588], "size": 5}
r = requests.get(url, headers=H, params=params, timeout=10)
data = r.json()
# Check for cursor/next fields
print(f"  Response keys: {[k for k in data.keys() if k != 'data']}")
if data.get("data"):
    last_id = data["data"][-1].get("id")
    print(f"  Last ID: {last_id}")
    # Try after param
    params2 = {"feedIds[]": [18588], "size": 5, "after": last_id}
    r2 = requests.get(url, headers=H, params=params2, timeout=10)
    data2 = r2.json()
    prods2 = data2.get("data", [])
    if prods2:
        print(f"  after={last_id}: {prods2[0].get('id')} | {prods2[0].get('title','')[:40]}")
        if prods2[0].get("id") != data["data"][0].get("id"):
            print("  ✅ Cursor pagination works!")
        else:
            print("  ❌ Same data returned")



import requests
import sys
sys.path.insert(0, r'C:\Users\Roel\clawd\scripts\demandgen-pipeline')
from config import WEBGAINS_API, PUBLISHER_ID, webgains_headers

# Try size=10000 for ladify
cid = 1622250
url = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{cid}/feeds/products'
params = {'size': 10000, 'fields[]': ['title','brand','price','id','ean','colour','image_link','link','description','rrp','currency','in_stock','category_name','model_number']}
r = requests.get(url, headers=webgains_headers(), params=params)
d = r.json()
products = d.get('data', [])
print(f'Ladify: {len(products)} products returned (total: {d.get("pagination",{}).get("total")})')

brands = {}
has_data = 0
for p in products:
    b = p.get('brand', 'Unknown')
    brands[b] = brands.get(b, 0) + 1
    if p.get('ean'):
        has_data += 1

print(f'Products with EAN: {has_data}')
print('Brands:')
for b, c in sorted(brands.items(), key=lambda x: -x[1]):
    print(f'  {b:25}: {c}')

# Show sample products
print('\nSample products:')
seen_titles = set()
for p in products[:20]:
    t = p.get('title', '')[:60]
    if t not in seen_titles:
        seen_titles.add(t)
        print(f'  {p.get("brand",""):15} | {t:60} | {p.get("price",""):12} | EAN:{p.get("ean","")}')

# Also try manify with size=10000
cid2 = 1622230
url2 = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{cid2}/feeds/products'
r2 = requests.get(url2, headers=webgains_headers(), params=params)
d2 = r2.json()
products2 = d2.get('data', [])
print(f'\nManify: {len(products2)} products returned')



import requests
import json

awin_token = 'bebd7839-013a-434d-a3da-7db2ecefb6b1'
publisher_id = 374301
headers = {'Authorization': f'Bearer {awin_token}'}

# Get all joined programmes, look for sneaker shops
r = requests.get(f'https://api.awin.com/publishers/{publisher_id}/programmes?relationship=joined', headers=headers)
programmes = r.json()
print(f'Total joined programmes: {len(programmes)}')

sneaker_keywords = ['sneaker', 'shoe', 'schoen', 'footwear', 'sport', 'nike', 'adidas', 'puma', 'asics', 'zalando', 'jd', 'footlocker', 'snipes', 'size?']
for p in programmes:
    name = p.get('name', '')
    desc = p.get('description', '')[:100]
    combined = f'{name} {desc}'.lower()
    if any(kw in combined for kw in sneaker_keywords):
        print(f'  {p["id"]:>8} | {name[:40]:40} | {p.get("status","")}')

# Also try productdata download with specific advertiser
print('\n--- Trying product feed download ---')
# Try different format for datafeed
urls = [
    f'https://productdata.awin.com/datafeed/list/apikey/{awin_token}/',
    f'https://productdata.awin.com/datafeed/list/apikey/{awin_token}/language/nl/',
]
for url in urls:
    r = requests.get(url, timeout=15)
    print(f'{r.status_code}: {url[:80]}')
    if r.ok:
        print(f'  {r.text[:300]}')



#!/usr/bin/env python3
"""
SneakerPicks Feed Sync v2 — Cursor-based pagination
Fetches ALL products from Webgains feeds using search_cursor pagination.
"""
import requests
import psycopg2
import psycopg2.extras
import re
import hashlib
import time
import sys

# ── Config ──
WEBGAINS_API = "https://platform-api.webgains.com"
PUBLISHER_ID = "1347930"
WEBGAINS_TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"

DB_URL = "postgresql://supabase_admin:HzWrn2rMMxUYWO6OT5X0PScIVEvSjQMup9YSLIrkPQ@172.18.0.5:5432/sneakerpicks"

CAMPAIGNS = {
    "manify.nl":   "1622230",
    "ladify.nl":   "1622250",
    "thehike.nl":  "1622240",
    "mellowed.nl": "1622260",
}

HEADERS = {
    "Authorization": f"Bearer {WEBGAINS_TOKEN}",
    "Content-Type": "application/json",
    "Accept": "application/json",
}

PRODUCT_FIELDS = ["title", "image_link", "link", "price", "brand", "description", "id",
                   "sale_price", "gtin", "mpn", "colour", "size", "availability", "condition",
                   "product_type", "google_product_category"]


def slugify(text):
    text = text.lower().strip()
    text = re.sub(r'[^\w\s-]', '', text)
    text = re.sub(r'[\s_]+', '-', text)
    text = re.sub(r'-+', '-', text)
    return text[:200]


def parse_price(val):
    if not val:
        return None
    val = str(val).replace(',', '.').strip()
    match = re.search(r'[\d.]+', val)
    if match:
        try:
            return float(match.group())
        except:
            return None
    return None


def get_feeds():
    """Get all unique feeds across all campaigns."""
    all_feeds = {}
    for site, camp_id in CAMPAIGNS.items():
        url = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{camp_id}/feeds"
        try:
            r = requests.get(url, headers=HEADERS, params={"size": 100}, timeout=30)
            r.raise_for_status()
            data = r.json()
            for f in data.get("data", []):
                fid = f["id"]
                if fid not in all_feeds:
                    all_feeds[fid] = {
                        "id": fid,
                        "name": f.get("name", ""),
                        "program_name": f.get("program", {}).get("name", ""),
                        "program_id": str(f.get("program", {}).get("id", "")),
                        "product_count": f.get("products_successfully_stored", 0),
                        "campaign_id": camp_id,
                    }
            print(f"  {site} ({camp_id}): {len(data.get('data', []))} feeds")
        except Exception as e:
            print(f"  {site} ERROR: {e}")
    return all_feeds


def fetch_all_products_cursor(campaign_id, feed_id, max_pages=200):
    """Fetch ALL products using cursor-based pagination."""
    url = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{campaign_id}/feeds/products"
    all_products = []
    seen_ids = set()
    cursor = None
    
    for page in range(max_pages):
        params = {
            "feeds[]": [feed_id],
            "size": 1000,
            "fields[]": PRODUCT_FIELDS,
        }
        if cursor:
            params["search_cursor"] = cursor
        
        try:
            r = requests.get(url, headers=HEADERS, params=params, timeout=60)
            r.raise_for_status()
            data = r.json()
        except Exception as e:
            print(f"    Page {page} error: {e}")
            break
        
        products = data.get("data", [])
        if not products:
            break
        
        pagination = data.get("pagination", {})
        new_cursor = pagination.get("search_cursor")
        
        # Deduplicate
        new_products = []
        for p in products:
            pid = p.get("id")
            if pid and pid not in seen_ids:
                seen_ids.add(pid)
                new_products.append(p)
        
        if not new_products:
            print(f"    Page {page}: no new products, done")
            break
        
        all_products.extend(new_products)
        
        if page % 10 == 0 or len(new_products) < 900:
            print(f"    Page {page}: +{len(new_products)} new (total: {len(all_products)})")
        
        if new_cursor == cursor or not new_cursor:
            print(f"    Cursor stopped changing at page {page}")
            break
        
        cursor = new_cursor
        time.sleep(0.3)  # Rate limit
    
    return all_products


def main():
    print("=== SneakerPicks Feed Sync v2 (Cursor Pagination) ===")
    print()
    
    conn = psycopg2.connect(DB_URL)
    conn.autocommit = False
    cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    
    # Clear old data for clean import
    print("Clearing old data...")
    cur.execute("TRUNCATE product_offers, offers, price_history, featured_products, collection_products, collection_overrides, product_categories, product_tags, price_alerts, products, feeds, sync_logs, field_mappings RESTART IDENTITY CASCADE")
    cur.execute("DELETE FROM brands WHERE slug != 'unknown'")
    conn.commit()
    print("Done.\n")
    
    cur.execute("SELECT id FROM providers WHERE slug = 'webgains'")
    provider_id = cur.fetchone()["id"]
    
    print("Fetching feeds from Webgains API...")
    feeds = get_feeds()
    print(f"Found {len(feeds)} unique feeds\n")
    
    # Brand cache
    brand_cache = {}
    cur.execute("SELECT id, slug FROM brands")
    for row in cur.fetchall():
        brand_cache[row["slug"]] = row["id"]
    
    # Product slug cache to avoid duplicates
    slug_cache = set()
    # EAN to product_id cache
    ean_cache = {}
    
    total_products = 0
    total_offers = 0
    
    for feed_ext_id, feed_info in sorted(feeds.items()):
        fname = feed_info["name"]
        pcount = feed_info["product_count"]
        camp_id = feed_info["campaign_id"]
        
        print(f"\n{'='*60}")
        print(f"Feed: {fname} (ext_id={feed_ext_id}, ~{pcount} products)")
        print(f"{'='*60}")
        
        # Create feed in DB
        cur.execute("""
            INSERT INTO feeds (provider_id, external_id, name, merchant_name, product_count)
            VALUES (%s, %s, %s, %s, %s) RETURNING id
        """, (provider_id, str(feed_ext_id), fname, feed_info["program_name"], pcount or 0))
        db_feed_id = cur.fetchone()["id"]
        conn.commit()
        
        # Fetch ALL products with cursor pagination
        products = fetch_all_products_cursor(camp_id, feed_ext_id)
        if not products:
            print("  No products, skipping")
            continue
        
        print(f"  Total fetched: {len(products)}")
        
        feed_new = 0
        feed_offers = 0
        batch_size = 500
        
        for i in range(0, len(products), batch_size):
            batch = products[i:i+batch_size]
            
            for p in batch:
                title = (p.get("title") or "").strip()
                brand_name = (p.get("brand") or "").strip()
                image = (p.get("image_link") or "").strip()
                link = (p.get("link") or "").strip()
                price = parse_price(p.get("sale_price") or p.get("price"))
                original_price = parse_price(p.get("price")) if p.get("sale_price") else None
                ext_id = str(p.get("id", "")).strip()
                gtin_val = p.get("gtin") or ""
                if isinstance(gtin_val, list):
                    gtin_val = gtin_val[0] if gtin_val else ""
                ean = str(gtin_val).strip() or None
                colour = (p.get("colour") or "").strip()
                
                if not title or not link or not price:
                    continue
                
                # Brand
                brand_slug = slugify(brand_name) if brand_name else "unknown"
                if brand_slug not in brand_cache:
                    try:
                        cur.execute("""
                            INSERT INTO brands (name, slug) VALUES (%s, %s)
                            ON CONFLICT (slug) DO UPDATE SET name = EXCLUDED.name
                            RETURNING id
                        """, (brand_name or "Unknown", brand_slug))
                        brand_cache[brand_slug] = cur.fetchone()["id"]
                    except Exception:
                        conn.rollback()
                        cur.execute("SELECT id FROM brands WHERE slug = %s", (brand_slug,))
                        row = cur.fetchone()
                        if row:
                            brand_cache[brand_slug] = row["id"]
                        else:
                            continue
                brand_id = brand_cache[brand_slug]
                
                # Check EAN match
                product_id = None
                if ean and ean in ean_cache:
                    product_id = ean_cache[ean]
                elif ean:
                    cur.execute("SELECT id FROM products WHERE ean = %s", (ean,))
                    row = cur.fetchone()
                    if row:
                        product_id = row["id"]
                        ean_cache[ean] = product_id
                
                if not product_id:
                    # Generate unique slug
                    base_slug = slugify(f"{brand_name}-{title}")[:180]
                    slug_hash = hashlib.md5(f"{feed_ext_id}-{ext_id}".encode()).hexdigest()[:8]
                    product_slug = f"{base_slug}-{slug_hash}"
                    
                    if product_slug in slug_cache:
                        # Already created this product, get its ID
                        cur.execute("SELECT id FROM products WHERE slug = %s", (product_slug,))
                        row = cur.fetchone()
                        if row:
                            product_id = row["id"]
                    
                    if not product_id:
                        try:
                            cur.execute("""
                                INSERT INTO products (name, slug, brand_id, brand, image_url, description, ean,
                                                      colorway, brand_normalized, first_seen_at)
                                VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, NOW())
                                RETURNING id
                            """, (title, product_slug, brand_id, brand_name, image,
                                  (p.get("description") or "")[:500], ean, colour,
                                  brand_name.lower() if brand_name else None))
                            product_id = cur.fetchone()["id"]
                            slug_cache.add(product_slug)
                            if ean:
                                ean_cache[ean] = product_id
                            feed_new += 1
                        except psycopg2.errors.UniqueViolation:
                            conn.rollback()
                            cur.execute("SELECT id FROM products WHERE slug = %s", (product_slug,))
                            row = cur.fetchone()
                            if row:
                                product_id = row["id"]
                            else:
                                continue
                        except Exception as e:
                            conn.rollback()
                            continue
                
                # Insert offer
                try:
                    cur.execute("""
                        INSERT INTO product_offers (product_id, feed_id, external_id, price, original_price,
                                                     currency, product_url, image_url, in_stock, last_seen_at)
                        VALUES (%s, %s, %s, %s, %s, 'EUR', %s, %s, true, NOW())
                        ON CONFLICT (feed_id, external_id) DO UPDATE SET
                            price = EXCLUDED.price,
                            original_price = EXCLUDED.original_price,
                            product_url = EXCLUDED.product_url,
                            image_url = EXCLUDED.image_url,
                            in_stock = true,
                            last_seen_at = NOW(),
                            updated_at = NOW()
                    """, (product_id, db_feed_id, ext_id, price, original_price, link, image))
                    feed_offers += 1
                except Exception as e:
                    conn.rollback()
                    continue
            
            # Commit per batch
            conn.commit()
        
        # Update feed stats
        cur.execute("""
            UPDATE feeds SET product_count = %s, last_sync_at = NOW(), last_sync_status = 'success'
            WHERE id = %s
        """, (feed_offers, db_feed_id))
        conn.commit()
        
        total_products += feed_new
        total_offers += feed_offers
        print(f"  ✓ {feed_new} new products, {feed_offers} offers")
    
    print(f"\n{'='*60}")
    print(f"SYNC COMPLETE")
    print(f"{'='*60}")
    print(f"New products created: {total_products}")
    print(f"Total offers: {total_offers}")
    
    cur.execute("SELECT COUNT(*) FROM products")
    print(f"Products in DB: {cur.fetchone()[0]}")
    cur.execute("SELECT COUNT(*) FROM product_offers")
    print(f"Offers in DB: {cur.fetchone()[0]}")
    cur.execute("SELECT COUNT(*) FROM brands")
    print(f"Brands in DB: {cur.fetchone()[0]}")
    cur.execute("SELECT COUNT(*) FROM feeds")
    print(f"Feeds in DB: {cur.fetchone()[0]}")
    cur.execute("SELECT brand, COUNT(*) c FROM products GROUP BY brand ORDER BY c DESC LIMIT 20")
    print("\nTop brands:")
    for row in cur.fetchall():
        print(f"  {row[0] or '(empty)'}: {row[1]}")
    
    cur.close()
    conn.close()


if __name__ == "__main__":
    main()



import requests

awin_token = 'bebd7839-013a-434d-a3da-7db2ecefb6b1'
headers = {'Authorization': f'Bearer {awin_token}'}

# Try different Awin API endpoints
endpoints = [
    ('Accounts', 'https://api.awin.com/accounts'),
    ('Publisher account', 'https://api.awin.com/publishers/me'),
    ('Publisher programmes', f'https://api.awin.com/publishers/me/programmes'),
    ('Datafeed list', f'https://productdata.awin.com/datafeed/list/apikey/{awin_token}'),
]

for name, url in endpoints:
    try:
        r = requests.get(url, headers=headers, timeout=15)
        print(f'{name}: {r.status_code}')
        if r.ok:
            text = r.text[:500]
            print(f'  {text}')
        else:
            print(f'  {r.text[:200]}')
    except Exception as e:
        print(f'{name}: ERROR - {e}')
    print()



import requests
import sys
sys.path.insert(0, r'C:\Users\Roel\clawd\scripts\demandgen-pipeline')
from config import WEBGAINS_API, PUBLISHER_ID, webgains_headers

# Try without campaign_id - publisher-level
url = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/feeds/products'
params = {'size': 5, 'fields[]': ['title','brand','price','id']}
r = requests.get(url, headers=webgains_headers(), params=params)
print('Publisher-level:', r.status_code)
if r.ok:
    d = r.json()
    for p in d.get('data', []):
        print(p.get('brand', ''), '|', p.get('title', ''))
else:
    print(r.text[:200])

print('---')
# Try thehike campaign (1622240)
url2 = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/1622240/feeds/products'
r2 = requests.get(url2, headers=webgains_headers(), params=params)
print('TheHike campaign:', r2.status_code)
if r2.ok:
    d2 = r2.json()
    for p in d2.get('data', []):
        print(p.get('brand', ''), '|', p.get('title', ''))
else:
    print(r2.text[:200])

print('---')
# Try POST instead of GET for manify
url3 = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/1622230/feeds/products'
body = {'size': 5, 'feeds': [18588], 'fields': ['title','brand','price','id']}
r3 = requests.post(url3, headers=webgains_headers(), json=body)
print('POST:', r3.status_code)
if r3.ok:
    d3 = r3.json()
    for p in d3.get('data', []):
        print(p.get('brand', ''), '|', p.get('title', ''))
else:
    print(r3.text[:200])

print('---')
# Try with sort parameter
params4 = {'size': 5, 'fields[]': ['title','brand','price','id'], 'sort': 'id', 'order': 'asc'}
r4 = requests.get(url3, headers=webgains_headers(), params=params4)
print('Sorted by id asc:', r4.status_code)
if r4.ok:
    d4 = r4.json()
    for p in d4.get('data', []):
        print(p.get('brand', ''), '|', p.get('title', ''), '| id:', p.get('id', ''))



#!/usr/bin/env python3
"""Try the products endpoint with feed filter - this is what feed-sync-v3 uses."""
import requests
import json

WEBGAINS_API = "https://platform-api.webgains.com"
PUBLISHER_ID = "1347930"
TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"

HEADERS = {
    "Authorization": f"Bearer {TOKEN}",
    "Accept": "application/json",
}

CAMP_ID = "1622230"

# The working endpoint from feed-sync-v3 - fetch products filtered by specific feed
# Baskets feed = 18588, Sneakermood = 16158
for feed_id, name in [("18588", "Baskets"), ("16158", "Sneakermood")]:
    url = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{CAMP_ID}/feeds/products"
    params = {
        "feeds[]": [feed_id],
        "size": 2,
        "fields[]": ["title", "brand", "link", "price", "google_product_category", "product_type", "gtin", "size", "colour", "id"],
    }
    print(f"\n=== Feed {name} ({feed_id}) ===")
    r = requests.get(url, headers=HEADERS, params=params, timeout=15)
    print(f"Status: {r.status_code}")
    data = r.json()
    products = data.get("data", [])
    print(f"Products returned: {len(products)}")
    for p in products:
        print(f"\n  Title: {p.get('title')}")
        print(f"  Brand: {p.get('brand')}")
        print(f"  Price: {p.get('price')}")
        print(f"  GPC: {p.get('google_product_category')}")
        print(f"  Product Type: {p.get('product_type')}")
        print(f"  GTIN: {p.get('gtin')}")
        print(f"  Size: {p.get('size')}")
        print(f"  Link: {str(p.get('link', ''))[:120]}")
        # Check which wgprogramid is in the link
        link = p.get('link', '')
        if 'wgprogramid=' in str(link):
            pgm = str(link).split('wgprogramid=')[1].split('&')[0]
            print(f"  → wgprogramid in link: {pgm}")



#!/usr/bin/env python3
"""Patch feed-sync-v4.py to fix category exclusion bug."""
import re

with open('/var/www/sneakerpicks/scripts/feed-sync-v4.py', 'r') as f:
    code = f.read()

# Fix is_sneaker: check footwear category IDs and inclusions BEFORE category exclusions
old_func = '''def is_sneaker(title, brand, product_type, google_category):
    title_lower = (title or '').lower()
    brand_lower = (brand or '').lower()
    ptype_lower = (product_type or '').lower()
    gcat_lower = (google_category or '').lower()

    for pat in EXCLUDE_TITLE_PATTERNS:
        if re.search(pat, title_lower):
            return False, f'title_exclude:{pat}'
    for pat in NON_SNEAKER_CATEGORY_PATTERNS:
        if pat in ptype_lower or pat in gcat_lower:
            return False, f'category_exclude:{pat}'
    for cat_id in FOOTWEAR_CATEGORY_IDS:
        if gcat_lower.startswith(cat_id) or f'> {cat_id}' in gcat_lower:
            return True, f'gcat_id:{cat_id}'
    for pat in SNEAKER_CATEGORY_PATTERNS:
        if pat in ptype_lower or pat in gcat_lower:
            return True, f'category_match:{pat}'
    if brand_lower in SNEAKER_BRANDS:
        return True, f'brand_match:{brand_lower}'
    for pat in SNEAKER_MODEL_PATTERNS:
        if re.search(pat, title_lower):
            return True, f'model_match:{pat}'
    return False, 'no_match\''''

new_func = '''def is_sneaker(title, brand, product_type, google_category):
    title_lower = (title or '').lower()
    brand_lower = (brand or '').lower()
    ptype_lower = (product_type or '').lower()
    gcat_lower = (google_category or '').lower()

    # Step 1: Title exclusions (clothing, accessories, etc.)
    for pat in EXCLUDE_TITLE_PATTERNS:
        if re.search(pat, title_lower):
            return False, f'title_exclude:{pat}'

    # Step 2: Google category ID check (187=Shoes, 1604=Athletic Shoes) — highest priority
    for cat_id in FOOTWEAR_CATEGORY_IDS:
        if gcat_lower.startswith(cat_id) or f'> {cat_id}' in gcat_lower:
            return True, f'gcat_id:{cat_id}'

    # Step 3: Category inclusions (sneaker/footwear/shoes in product_type or category)
    for pat in SNEAKER_CATEGORY_PATTERNS:
        if pat in ptype_lower or pat in gcat_lower:
            return True, f'category_match:{pat}'

    # Step 4: Category exclusions — only checked AFTER inclusions
    # This prevents "Apparel & Accessories > Shoes" from being excluded by 'accessori'
    for pat in NON_SNEAKER_CATEGORY_PATTERNS:
        if pat in ptype_lower or pat in gcat_lower:
            return False, f'category_exclude:{pat}'

    # Step 5: Known sneaker brand
    if brand_lower in SNEAKER_BRANDS:
        return True, f'brand_match:{brand_lower}'

    # Step 6: Known model patterns
    for pat in SNEAKER_MODEL_PATTERNS:
        if re.search(pat, title_lower):
            return True, f'model_match:{pat}'

    return False, 'no_match\''''

if old_func in code:
    code = code.replace(old_func, new_func)
    with open('/var/www/sneakerpicks/scripts/feed-sync-v4.py', 'w') as f:
        f.write(code)
    print("PATCHED successfully")
else:
    print("ERROR: old function not found, trying to find it...")
    # Show surrounding context
    idx = code.find('def is_sneaker')
    if idx >= 0:
        print(code[idx:idx+800])
    else:
        print("is_sneaker not found at all!")



#!/usr/bin/env python3
"""Discover Awin feeds for sneaker shops."""
import requests
import json

TOKEN = 'bebd7839-013a-434d-a3da-7db2ecefb6b1'
PUB_ID = '374301'
BASE = 'https://productdata.awin.com'

# 1. List all advertisers we're joined with
print("=== Awin Advertisers ===")
r = requests.get(f'https://api.awin.com/publishers/{PUB_ID}/programmes?relationship=joined',
                 headers={'Authorization': f'Bearer {TOKEN}'}, timeout=30)
print(f'Status: {r.status_code}')
if r.status_code == 200:
    programs = r.json()
    print(f'Total joined programmes: {len(programs)}')
    for p in programs:
        name = p.get('name', '?')
        pid = p.get('id', '?')
        keywords = ['sneaker', 'shoe', 'schoen', 'sport', 'nike', 'adidas', 'puma', 'asics',
                    'foot', 'jd ', 'zalando', 'snipes', 'size?', 'otrium', 'omoda']
        is_relevant = any(kw in name.lower() for kw in keywords)
        if is_relevant:
            print(f'  *** [{pid}] {name}')
        # Print all to see what's available
    # Print all
    for p in programs:
        print(f'  [{p.get("id")}] {p.get("name")}')
else:
    print(f'Error: {r.text[:500]}')

# 2. List available product feeds/datafeeds
print("\n=== Awin Datafeeds ===")
r2 = requests.get(f'{BASE}/feeds/list?publisherId={PUB_ID}',
                  headers={'Authorization': f'Bearer {TOKEN}'}, timeout=30)
print(f'Status: {r2.status_code}')
if r2.status_code == 200:
    try:
        feeds = r2.json()
        print(f'Total feeds: {len(feeds)}')
        for f in feeds[:50]:
            print(f'  [{f.get("advertiserId")}] {f.get("advertiserName", "?")} - {f.get("feedName", "?")} [{f.get("productCount", "?")} products]')
    except:
        print(f'Response (not JSON): {r2.text[:1000]}')
else:
    print(f'Error: {r2.text[:500]}')

# 3. Try the product search endpoint  
print("\n=== Awin Product Search ===")
r3 = requests.get(f'{BASE}/product-feeds?publisherId={PUB_ID}',
                  headers={'Authorization': f'Bearer {TOKEN}'}, timeout=30)
print(f'Status: {r3.status_code}')
if r3.status_code == 200:
    print(f'Response: {r3.text[:2000]}')
else:
    print(f'Error: {r3.text[:500]}')

# 4. Try getting feeds via commission groups/programmes endpoint
print("\n=== Trying product list endpoint ===")
r4 = requests.get(f'https://api.awin.com/publishers/{PUB_ID}/product-feeds',
                  headers={'Authorization': f'Bearer {TOKEN}'}, timeout=30)
print(f'Status: {r4.status_code}')
if r4.status_code == 200:
    print(f'Response: {r4.text[:2000]}')
else:
    print(f'Error: {r4.text[:500]}')



#!/usr/bin/env python3
"""Fix size dedup issues in feed-sync-v4.py:
1. Strip "- Maat XX - Material" from Foot Locker names
2. Prevent multiple offers per product per feed (merge sizes)
3. Better name normalization
"""

with open('/var/www/sneakerpicks/scripts/feed-sync-v4.py', 'r') as f:
    code = f.read()

# 1. Fix normalize_product_name to strip "- Maat XX" patterns from middle/end
old_norm = '''def normalize_product_name(title, brand):
    name = re.sub(r'\\s+(?:eu|eur|maat|size)\\s*\\d{2}(?:[.,]\\d{1,2})?\\s*$', '', title.strip(), flags=re.I)
    name = re.sub(r'\\s+\\d{2}(?:[.,]\\d{1,2})?\\s*$', '', name)
    # Remove size like "- US3.0" or "- EU 42" from end
    name = re.sub(r'\\s*[-/]\\s*(?:US|EU)\\s*\\d+(?:\\.\\d+)?\\s*$', '', name, flags=re.I)
    return name.strip(' -')'''

new_norm = '''def normalize_product_name(title, brand):
    name = title.strip()
    # Strip "- Maat XX" or "- Maat XX 2/3" from anywhere (Foot Locker pattern)
    name = re.sub(r'\\s*-\\s*Maat\\s+\\d+(?:\\s+\\d+/\\d+|[.,]\\d+)?\\s*', ' ', name, flags=re.I)
    # Strip "- Material" suffix like "- Leer", "- Mesh/Synthetisch", "- Textil"
    name = re.sub(r'\\s*-\\s*(?:Leer|Mesh[/\\w]*|Textil|Synthetisch|Suede|Canvas|Nubuck|Rubber)\\s*$', '', name, flags=re.I)
    # Strip "EU XX", "Maat XX", "Size XX" from end
    name = re.sub(r'\\s+(?:eu|eur|maat|size)\\s*\\d{2}(?:[.,]\\d{1,2})?\\s*$', '', name, flags=re.I)
    # Strip bare size number from end
    name = re.sub(r'\\s+\\d{2}(?:[.,]\\d{1,2})?\\s*$', '', name)
    # Strip "- US3.0,EU42.3,UK9.0" (Patta pattern)
    name = re.sub(r'\\s*-\\s*US\\d+[.,]\\d+.*$', '', name, flags=re.I)
    # Strip "- US3.0" or "- EU 42" from end
    name = re.sub(r'\\s*[-/]\\s*(?:US|EU)\\s*\\d+(?:\\.\\d+)?\\s*$', '', name, flags=re.I)
    # Clean up double spaces and trailing dashes
    name = re.sub(r'\\s+', ' ', name).strip(' -')
    return name'''

if old_norm in code:
    code = code.replace(old_norm, new_norm)
    print("PATCHED: normalize_product_name")
else:
    print("WARNING: Could not find normalize_product_name to patch")

# 2. Add a dedup step after all offers are created: merge offers with same product_id+feed_id
# We'll add this as a post-processing function and call it at the end of each shop's processing

dedup_func = '''
def merge_duplicate_offers(cur, conn, db_feed_id, shop_name):
    """Merge offers that point to the same product from the same feed.
    This happens when different group keys resolve to the same product via EAN/fingerprint.
    Keeps the offer with the most sizes, merges sizes from others, then deletes dupes."""
    cur.execute("""
        SELECT product_id, array_agg(id ORDER BY jsonb_array_length(sizes) DESC) as offer_ids,
               COUNT(*) as cnt
        FROM product_offers WHERE feed_id = %s
        GROUP BY product_id HAVING COUNT(*) > 1
    """, (db_feed_id,))
    
    dupes = cur.fetchall()
    if not dupes:
        return 0
    
    merged = 0
    for row in dupes:
        product_id = row['product_id']
        offer_ids = row['offer_ids']
        keep_id = offer_ids[0]  # the one with most sizes
        delete_ids = offer_ids[1:]
        
        # Collect all sizes from all offers
        cur.execute("""
            SELECT sizes, price, original_price FROM product_offers 
            WHERE id = ANY(%s) ORDER BY jsonb_array_length(sizes) DESC
        """, (offer_ids,))
        
        all_sizes = []
        seen_size_keys = set()
        best_price = None
        best_orig = None
        
        for offer_row in cur.fetchall():
            import json
            sizes = offer_row['sizes'] if isinstance(offer_row['sizes'], list) else json.loads(offer_row['sizes'] or '[]')
            price = offer_row['price']
            orig = offer_row['original_price']
            
            if best_price is None or (price and price < best_price):
                best_price = price
            if orig and (best_orig is None or orig < best_orig):
                best_orig = orig
                
            for s in sizes:
                size_key = s.get('size', '') + '|' + s.get('ean', '')
                if size_key not in seen_size_keys:
                    seen_size_keys.add(size_key)
                    all_sizes.append(s)
        
        # Update the keeper with merged sizes
        import json
        cur.execute("""
            UPDATE product_offers SET sizes = %s::jsonb, price = %s, original_price = %s
            WHERE id = %s
        """, (json.dumps(all_sizes, ensure_ascii=False), best_price, best_orig, keep_id))
        
        # Delete the duplicates
        cur.execute("DELETE FROM product_offers WHERE id = ANY(%s)", (delete_ids,))
        merged += len(delete_ids)
    
    conn.commit()
    if merged > 0:
        print(f"  Merged {merged} duplicate offers for {len(dupes)} products")
    return merged
'''

# Insert the function before "# ── Main ──"
main_marker = "\n# ── Main ──"
main_idx = code.find(main_marker)
if main_idx >= 0:
    code = code[:main_idx] + dedup_func + code[main_idx:]
    print("PATCHED: Added merge_duplicate_offers function")
else:
    print("WARNING: Could not find Main section")

# 3. Call merge_duplicate_offers after each shop's offers are committed
# For Webgains shops
old_wg_end = '''        total_new += new_products; total_offers += shop_offers; total_ph += ph_count
        print(f"  => {new_products} new products, {shop_offers} offers, {ph_count} price records")

    # ── Awin Feeds ──'''

new_wg_end = '''        # Merge duplicate offers (same product, same feed)
        merged = merge_duplicate_offers(cur, conn, db_feed_id, shop_name)
        shop_offers -= merged

        total_new += new_products; total_offers += shop_offers; total_ph += ph_count
        print(f"  => {new_products} new products, {shop_offers} offers, {ph_count} price records")

    # ── Awin Feeds ──'''

if old_wg_end in code:
    code = code.replace(old_wg_end, new_wg_end)
    print("PATCHED: Added merge call for Webgains shops")
else:
    print("WARNING: Could not find Webgains end section to patch")

# For Awin shops - find the similar pattern
old_aw_end = '''        total_new += new_products; total_offers += shop_offers; total_ph += ph_count
        print(f"  => {new_products} new products, {shop_offers} offers, {ph_count} price records")

    # Summary'''

new_aw_end = '''        # Merge duplicate offers (same product, same feed)
        merged = merge_duplicate_offers(cur, conn, db_feed_id, shop_name)
        shop_offers -= merged

        total_new += new_products; total_offers += shop_offers; total_ph += ph_count
        print(f"  => {new_products} new products, {shop_offers} offers, {ph_count} price records")

    # Summary'''

if old_aw_end in code:
    code = code.replace(old_aw_end, new_aw_end)
    print("PATCHED: Added merge call for Awin shops")
else:
    print("WARNING: Could not find Awin end section to patch")

with open('/var/www/sneakerpicks/scripts/feed-sync-v4.py', 'w') as f:
    f.write(code)

print("\nAll patches applied!")



#!/usr/bin/env python3
"""Check if Webgains feeds have download URLs"""
import requests, json

API = "https://platform-api.webgains.com"
PUB = "1347930"
TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
H = {"Authorization": f"Bearer {TOKEN}"}

# Get feeds from manify campaign
r = requests.get(f"{API}/auth/publishers/{PUB}/campaigns/1622230/feeds", headers=H, params={"size": 100})
feeds = r.json().get("data", [])

# Print ALL fields of first feed to find download URL
print("=== First feed - ALL fields ===")
print(json.dumps(feeds[0], indent=2, default=str))

print("\n=== All feeds summary ===")
for f in feeds:
    fid = f["id"]
    name = f.get("name", "")
    count = f.get("products_successfully_stored", 0)
    # Look for any URL fields
    urls = {k: v for k, v in f.items() if isinstance(v, str) and "http" in v}
    print(f"  {fid} | {name} | {count} products | urls: {urls}")

# Also try getting a single feed's detail
print("\n=== Single feed detail ===")
fid = feeds[0]["id"]
for endpoint in [
    f"{API}/auth/publishers/{PUB}/feeds/{fid}",
    f"{API}/auth/publishers/{PUB}/campaigns/1622230/feeds/{fid}",
    f"{API}/auth/publishers/{PUB}/campaigns/1622230/feeds/{fid}/download",
]:
    try:
        r = requests.get(endpoint, headers=H, timeout=10)
        print(f"\n{endpoint} -> {r.status_code}")
        if r.status_code == 200:
            ct = r.headers.get("content-type", "")
            if "json" in ct:
                print(json.dumps(r.json(), indent=2, default=str)[:1000])
            else:
                print(f"Content-Type: {ct}, size: {len(r.content)}")
                print(r.text[:500])
    except Exception as e:
        print(f"{endpoint} -> ERROR: {e}")



#!/usr/bin/env python3
"""Check which feeds provide images with transparency."""
import subprocess, json, urllib.request, tempfile, os, sys

# Get samples from DB
result = subprocess.run(
    ['docker', 'exec', '-i', 'supabase-db', 'psql', '-U', 'postgres', '-d', 'sneakerpicks', '-t', '-A', '-F', '|'],
    input="""
WITH ranked AS (
  SELECT f.merchant_name, f.id as feed_id, po.image_url,
    ROW_NUMBER() OVER (PARTITION BY f.id ORDER BY random()) as rn
  FROM feeds f
  JOIN product_offers po ON po.feed_id = f.id
  WHERE po.image_url IS NOT NULL AND po.image_url != ''
)
SELECT merchant_name, feed_id, image_url
FROM ranked
WHERE rn <= 5
ORDER BY feed_id, rn;
""",
    capture_output=True, text=True
)

lines = [l.strip() for l in result.stdout.strip().split('\n') if l.strip()]
print(f"Got {len(lines)} sample URLs")

try:
    from PIL import Image
    has_pil = True
except ImportError:
    has_pil = False
    print("WARNING: Pillow not installed, checking file headers only")

results = {}  # feed_id -> {merchant, total, png_count, alpha_count}

for line in lines:
    parts = line.split('|')
    if len(parts) < 3:
        continue
    merchant, feed_id, url = parts[0], parts[1], parts[2]
    
    if feed_id not in results:
        results[feed_id] = {'merchant': merchant, 'total': 0, 'png': 0, 'alpha': 0, 'jpg': 0}
    
    results[feed_id]['total'] += 1
    
    try:
        tmpfile = f"/tmp/transp_check_{feed_id}_{results[feed_id]['total']}"
        req = urllib.request.Request(url, headers={'User-Agent': 'Mozilla/5.0'})
        with urllib.request.urlopen(req, timeout=10) as resp:
            data = resp.read()
        with open(tmpfile, 'wb') as f:
            f.write(data)
        
        # Check magic bytes
        is_png = data[:4] == b'\x89PNG'
        is_jpg = data[:2] == b'\xff\xd8'
        
        if is_jpg:
            results[feed_id]['jpg'] += 1
        elif is_png:
            results[feed_id]['png'] += 1
            if has_pil:
                img = Image.open(tmpfile)
                if img.mode in ('RGBA', 'LA', 'PA'):
                    results[feed_id]['alpha'] += 1
        
        os.unlink(tmpfile)
    except Exception as e:
        print(f"  Error checking {url[:60]}...: {e}")

print("\n=== RESULTS ===")
print(f"{'Merchant':<25} {'Feed':>4} {'Total':>5} {'JPG':>4} {'PNG':>4} {'Alpha':>5}")
print("-" * 55)
for fid, r in sorted(results.items(), key=lambda x: x[1]['merchant']):
    print(f"{r['merchant']:<25} {fid:>4} {r['total']:>5} {r['jpg']:>4} {r['png']:>4} {r['alpha']:>5}")



import requests
import sys
sys.path.insert(0, r'C:\Users\Roel\clawd\scripts\demandgen-pipeline')
from config import WEBGAINS_API, PUBLISHER_ID, PROGRAMS, webgains_headers

camp_id = PROGRAMS['manify.nl']['campaign_id']
url = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{camp_id}/feeds/products'

# Test with Baskets feed (18588)
params = {'size': 5, 'feeds[]': [18588], 'fields[]': ['title','brand','price','id']}
r = requests.get(url, headers=webgains_headers(), params=params)
print('URL:', r.url[:300])
print('Status:', r.status_code)
d = r.json()
for p in d.get('data', []):
    print(p.get('brand', ''), '|', p.get('title', ''))
print('Pagination:', d.get('pagination'))
print('---')

# Page 2 using search_after
cursor = d.get('pagination', {}).get('search_cursor', '')
if cursor:
    params2 = {'size': 5, 'search_after': cursor, 'fields[]': ['title','brand','price','id']}
    r2 = requests.get(url, headers=webgains_headers(), params=params2)
    print('Page 2 URL:', r2.url[:300])
    d2 = r2.json()
    for p in d2.get('data', []):
        print(p.get('brand', ''), '|', p.get('title', ''))
    print('Pagination:', d2.get('pagination'))



#!/usr/bin/env python3
"""Patch Awin section of feed-sync-v4.py for transparent feeds."""

with open("/var/www/sneakerpicks/scripts/feed-sync-v4.py", "r") as f:
    content = f.read()

# Find the Awin offer section - the "ext_id = str(variants[0]" that comes after "# ── Awin feeds ──"
awin_marker = "# ── Awin feeds ──"
awin_pos = content.find(awin_marker)
if awin_pos == -1:
    print("ERROR: Could not find Awin section marker")
    exit(1)

# Find "ext_id = str(variants[0]" after the awin marker
target = '            ext_id = str(variants[0].get("id","")).strip()'
target_pos = content.find(target, awin_pos)
if target_pos == -1:
    print("ERROR: Could not find ext_id line in Awin section")
    exit(1)

patch = """            # Set cutout for transparent feeds (Awin)
            if product_id and shop_name in TRANSPARENT_FEED_NAMES and image and image.lower().endswith('.png'):
                try:
                    cur.execute("UPDATE products SET cutout_image_url = image_url, needs_cutout = false WHERE id = %s AND cutout_image_url IS NULL", (product_id,))
                except: conn.rollback()

"""

if "Set cutout for transparent feeds (Awin)" not in content:
    content = content[:target_pos] + patch + content[target_pos:]
    with open("/var/www/sneakerpicks/scripts/feed-sync-v4.py", "w") as f:
        f.write(content)
    print("Patched Awin section successfully")
else:
    print("Awin section already patched")



import requests

awin_token = 'bebd7839-013a-434d-a3da-7db2ecefb6b1'
publisher_id = 374301
headers = {'Authorization': f'Bearer {awin_token}'}

# Use correct publisher ID
endpoints = [
    ('Programmes', f'https://api.awin.com/publishers/{publisher_id}/programmes'),
    ('Programmes joined', f'https://api.awin.com/publishers/{publisher_id}/programmes?relationship=joined'),
    ('Datafeed list', f'https://productdata.awin.com/datafeed/list/apikey/{awin_token}/language/nl/fid//columns/aw_deep_link,product_name,aw_product_id,merchant_product_id,merchant_image_url,description,merchant_category,search_price,merchant_name,aw_image_url,brand_name,colour,rrp_price,ean,model_number,in_stock,currency/format/csv/delimiter/%2C/compression/gzip/adultcontent/1/'),
]

for name, url in endpoints:
    try:
        r = requests.get(url, headers=headers, timeout=30, stream=True)
        print(f'{name}: {r.status_code} ({r.headers.get("Content-Type","")})')
        if r.ok:
            content = next(r.iter_content(500))
            try:
                print(f'  {content.decode("utf-8", errors="replace")[:500]}')
            except:
                print(f'  Binary data, size: {r.headers.get("Content-Length", "?")}')
        else:
            print(f'  {r.text[:200]}')
    except Exception as e:
        print(f'{name}: ERROR - {e}')
    print()



#!/usr/bin/env python3
"""Test if Webgains feeds return different data per feed ID."""
import requests, json

WEBGAINS_API = "https://platform-api.webgains.com"
PUBLISHER_ID = "1347930"
CAMPAIGN_ID = "1622230"
TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
H = {"Authorization": f"Bearer {TOKEN}"}

# 1. Check feed metadata
print("=== Feed Metadata ===")
url = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{CAMPAIGN_ID}/feeds"
r = requests.get(url, headers=H, params={"size": 200}, timeout=30)
feeds = r.json().get("data", [])
target_ids = {18588, 25578, 23108, 36168, 35896, 35808, 35630, 16158, 26388, 34710}
for f in feeds:
    if f["id"] in target_ids:
        prog = f.get("program", {})
        print(f"  Feed {f['id']}: {f.get('name')} | program={prog.get('name')} | products={f.get('products_successfully_stored')}")

# 2. Compare product data between feeds
print("\n=== Product Comparison ===")
test_feeds = [18588, 16158, 25578]
first_ids = {}
for fid in test_feeds:
    url2 = f"{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{CAMPAIGN_ID}/feeds/products"
    r2 = requests.get(url2, headers=H, params={"feeds[]": [fid], "size": 5, "fields[]": ["title", "brand", "id"]}, timeout=30)
    data = r2.json()
    products = data.get("data", [])
    total = data.get("recordsTotal", "?")
    ids = [p.get("id") for p in products]
    first_ids[fid] = ids
    print(f"\nFeed {fid} (total={total}):")
    for p in products:
        print(f"  {p.get('id')} | {p.get('brand')} | {p.get('title','')[:60]}")

# 3. Check if feeds return identical data
print("\n=== Duplicate Check ===")
if first_ids.get(18588) == first_ids.get(16158):
    print("⚠️  FEEDS 18588 AND 16158 RETURN IDENTICAL DATA — API IS BROKEN!")
else:
    print("✅ Feeds return different data")

# 4. Try the feed download URL approach
print("\n=== Feed Download URLs ===")
for f in feeds:
    if f["id"] in {18588, 16158, 25578}:
        download_url = f.get("download_url") or f.get("url") or f.get("feed_url")
        print(f"  Feed {f['id']}: download_url={download_url}")
        # Print all keys
        print(f"    Keys: {list(f.keys())}")
        if f["id"] == 18588:
            print(f"    Full data: {json.dumps(f, indent=2, default=str)[:1000]}")



#!/usr/bin/env python3
"""Test cursor-based pagination with search_cursor."""
import requests, json

TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
PUBLISHER_ID = "1347930"
H = {"Authorization": f"Bearer {TOKEN}", "Accept": "application/json"}
url = f"https://platform-api.webgains.com/auth/publishers/{PUBLISHER_ID}/campaigns/1622230/feeds/products"

# Test cursor pagination with Sneakermood (1849 total, should need 2 pages)
print("=== Sneakermood cursor pagination ===")
all_ids = set()
cursor = None
for page in range(5):
    params = {"feedIds[]": [16158], "size": 1000}
    if cursor:
        params["search_cursor"] = cursor
    r = requests.get(url, headers=H, params=params, timeout=60)
    data = r.json()
    prods = data.get("data", [])
    pag = data.get("pagination", {})
    cursor = pag.get("search_cursor")
    
    new_ids = set(str(p.get("id")) for p in prods)
    overlap = all_ids & new_ids
    all_ids |= new_ids
    
    print(f"  Page {page+1}: {len(prods)} products, {len(overlap)} overlap, total unique: {len(all_ids)}, cursor={cursor}")
    
    if not prods or len(prods) < 1000:
        break
    if len(overlap) == len(new_ids):
        print("  All duplicates, stopping")
        break

print(f"\nTotal unique products: {len(all_ids)}")



#!/usr/bin/env python3
"""Parse Awin feed list and find sneaker-relevant NL feeds."""
import requests
import csv
import io

url = "https://ui.awin.com/productdata-darwin-download/publisher/374301/489bdc79c6362be4f7d60e6b2506a2de/1/feedList"
r = requests.get(url, timeout=30)
reader = csv.DictReader(io.StringIO(r.text))

sneaker_keywords = ['shoe', 'sneaker', 'footwear', 'sport', 'jd', 'foot locker', 'schuurman', 'bever', 'nike', 'adidas', 'puma', 'asics', 'vans', 'converse', 'new balance', 'zalando', 'omoda', 'nelson', 'van haren', 'sarenza', 'torfs']
nl_regions = ['NL', 'BE']

print("=== ALL FEEDS ===")
all_feeds = []
for row in reader:
    all_feeds.append(row)

print(f"Total feeds: {len(all_feeds)}")

# Filter for NL/BE region or sneaker-related
print(f"\n=== NL/BE FEEDS ===")
for row in sorted(all_feeds, key=lambda x: x.get('Advertiser Name', '')):
    region = row.get('Primary Region', '')
    name = row.get('Advertiser Name', '').lower()
    products = row.get('No of products', '0')
    status = row.get('Membership Status', '')
    feed_id = row.get('Feed ID', '')
    feed_url = row.get('URL', '')
    
    if region in nl_regions:
        print(f"  [{region}] {row['Advertiser Name']} ({products} products) - {status} - {feed_id}")
        print(f"    URL: {feed_url}")

print(f"\n=== SNEAKER-RELEVANT (any region) ===")
for row in sorted(all_feeds, key=lambda x: -int(x.get('No of products', '0'))):
    name = row.get('Advertiser Name', '').lower()
    region = row.get('Primary Region', '')
    products = row.get('No of products', '0')
    status = row.get('Membership Status', '')
    feed_id = row.get('Feed ID', '')
    
    if any(kw in name for kw in sneaker_keywords):
        print(f"  [{region}] {row['Advertiser Name']} ({products} products) - {status} - {feed_id}")



#!/usr/bin/env python3
"""Check Webgains 2.0 API for feed download."""
import requests

TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
PUBLISHER_ID = "1347930"

HEADERS = {"Authorization": f"Bearer {TOKEN}", "Accept": "application/json"}

# Check what the 2.0 API returns
r = requests.get("https://api.webgains.com/2.0/feeds/18588/download", headers=HEADERS, timeout=10)
print(f"2.0 Status: {r.status_code}")
print(f"2.0 Body: {r.text[:500]}")

# Try the old webgains feed export URL format
# Webgains traditionally uses: https://productdata.webgains.com/newDownloadFeed.html?feedid=XXXX
for url in [
    "http://productdata.webgains.com/newDownloadFeed.html?feedid=18588",
    "https://productdata.webgains.com/newDownloadFeed.html?feedid=18588",
    f"http://productdata.webgains.com/newDownloadFeed.html?feedid=18588&publisherId={PUBLISHER_ID}",
]:
    try:
        r = requests.get(url, headers=HEADERS, timeout=10, stream=True, allow_redirects=True)
        ct = r.headers.get('content-type', '')
        print(f"\n{r.status_code} {ct[:60]} {url}")
        if r.status_code == 200:
            chunk = next(r.iter_content(500))
            print(f"  FIRST BYTES: {chunk[:300]}")
        r.close()
    except Exception as e:
        print(f"\nERR: {e}")



#!/usr/bin/env python3
"""Check pagination response and try to get >1000 products."""
import requests, json

TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
PUBLISHER_ID = "1347930"
H = {"Authorization": f"Bearer {TOKEN}", "Accept": "application/json"}
url = f"https://platform-api.webgains.com/auth/publishers/{PUBLISHER_ID}/campaigns/1622230/feeds/products"

# Check pagination response
params = {"feedIds[]": [18588], "size": 5}
r = requests.get(url, headers=H, params=params, timeout=10)
data = r.json()
print("Pagination field:", json.dumps(data.get("pagination"), indent=2))

# Check: how many unique products can we get from 1000?
print("\n=== Get 1000 from each feed ===")
for fid, name in [(18588, "Baskets"), (16158, "Sneakermood"), (25578, "Patta")]:
    params = {"feedIds[]": [fid], "size": 1000}
    r = requests.get(url, headers=H, params=params, timeout=60)
    data = r.json()
    prods = data.get("data", [])
    unique_ids = len(set(str(p.get("id")) for p in prods))
    brands = {}
    categories = {}
    for p in prods:
        b = p.get("brand", "?")
        brands[b] = brands.get(b, 0) + 1
        cat = p.get("product_type")
        if isinstance(cat, list):
            cat = cat[0] if cat else "?"
        categories[str(cat)] = categories.get(str(cat), 0) + 1
    
    print(f"\n{name} (feed {fid}): {len(prods)} products, {unique_ids} unique")
    print(f"  Top brands: {sorted(brands.items(), key=lambda x: -x[1])[:5]}")
    print(f"  Categories: {sorted(categories.items(), key=lambda x: -x[1])[:10]}")
    
    # Pagination info
    pag = data.get("pagination", {})
    print(f"  Pagination: {pag}")

# Check if we can use search/filter within feedIds to paginate
print("\n=== Try search within feed ===")
params = {"feedIds[]": [18588], "size": 1000, "search": "a"}
r = requests.get(url, headers=H, params=params, timeout=30)
data = r.json()
print(f"Search 'a': {len(data.get('data',[]))} results")

params = {"feedIds[]": [18588], "size": 1000, "search": "nike"}
r = requests.get(url, headers=H, params=params, timeout=30)
data = r.json()
print(f"Search 'nike': {len(data.get('data',[]))} results")

params = {"feedIds[]": [18588], "size": 1000, "search": "adidas"}
r = requests.get(url, headers=H, params=params, timeout=30)
data = r.json()
print(f"Search 'adidas': {len(data.get('data',[]))} results")



#!/usr/bin/env python3
"""Debug: sample rows from Van Arendonk CSV to see why 0 sneakers matched."""
import requests, csv, io

TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"

FEEDS = [("17108", "Van Arendonk"), ("36168", "Patta"), ("18588", "Baskets")]
BASE = "https://platform-api.webgains.com/auth/publishers/1347930/campaigns/1622230/feeds/products"

for fid, name in FEEDS:
    url = f"{BASE}?feedIds[]={fid}&format=csv"
    r = requests.get(url, headers={"Authorization": f"Bearer {TOKEN}"}, timeout=60)
    reader = csv.DictReader(io.StringIO(r.text))
    rows = list(reader)
    
    print(f"\n=== {name} ({fid}) — {len(rows)} rows ===")
    print(f"Columns: {reader.fieldnames}")
    
    # Find sneaker-like rows
    sneaker_brands = {'nike', 'adidas', 'new balance', 'puma', 'asics', 'jordan', 'converse', 'vans'}
    found = 0
    for row in rows:
        brand = (row.get('brand') or '').lower()
        title = (row.get('title') or '').lower()
        gcat = row.get('google_product_category') or row.get('google_product_category_text') or ''
        ptype = row.get('product_type') or ''
        if brand in sneaker_brands or 'sneaker' in title:
            if found < 3:
                print(f"\n  Brand: {row.get('brand')}")
                print(f"  Title: {row.get('title')}")
                print(f"  google_product_category: {gcat}")
                print(f"  product_type: {ptype}")
                print(f"  google_product_category_text: {row.get('google_product_category_text', 'N/A')}")
            found += 1
    print(f"  Found {found} sneaker-brand rows out of {len(rows)}")



import requests
import sys
sys.path.insert(0, r'C:\Users\Roel\clawd\scripts\demandgen-pipeline')
from config import WEBGAINS_API, PUBLISHER_ID, webgains_headers

# ladify campaign showed sneaker products, let's explore it
cid = 1622250
url = f'{WEBGAINS_API}/auth/publishers/{PUBLISHER_ID}/campaigns/{cid}/feeds/products'

# Get first 10 products
params = {'size': 10, 'fields[]': ['title','brand','price','id','category_name','image_link','link','ean','colour','model_number','in_stock','rrp','currency','description']}
r = requests.get(url, headers=webgains_headers(), params=params)
d = r.json()
print(f'Total: {d.get("pagination", {}).get("total")}')
print(f'Cursor: {d.get("pagination", {}).get("search_cursor")}')
for p in d.get('data', []):
    print(f'{p.get("brand",""):15} | {p.get("title","")[:60]:60} | {p.get("price",""):12} | {p.get("category_name","")}')
    
# Try cursor pagination
cursor = d.get('pagination', {}).get('search_cursor', '')
if cursor:
    print(f'\n--- Page 2 (cursor: {cursor}) ---')
    params2 = {'size': 5, 'search_after': cursor, 'fields[]': ['title','brand','price','id']}
    r2 = requests.get(url, headers=webgains_headers(), params=params2)
    d2 = r2.json()
    print(f'Cursor: {d2.get("pagination", {}).get("search_cursor")}')
    for p in d2.get('data', []):
        print(f'{p.get("brand",""):15} | {p.get("title","")[:60]}')



#!/usr/bin/env python3
"""Test cursor-based pagination with Webgains"""
import requests, json

API = "https://platform-api.webgains.com"
PUB = "1347930"
TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
H = {"Authorization": f"Bearer {TOKEN}"}
CAMP = "1622230"
FEED = "17108"

url = f"{API}/auth/publishers/{PUB}/campaigns/{CAMP}/feeds/products"

all_ids = set()
cursor = None
for page in range(5):
    params = {
        "feeds[]": [FEED],
        "size": 1000,
        "fields[]": ["title", "id", "brand"],
    }
    if cursor:
        params["search_after"] = cursor
    
    r = requests.get(url, headers=H, params=params, timeout=60)
    data = r.json()
    pagination = data.get("pagination", {})
    products = data.get("data", [])
    ids = set(p["id"] for p in products)
    new_ids = ids - all_ids
    all_ids |= ids
    cursor = pagination.get("search_cursor")
    
    print(f"Page {page}: {len(products)} products, {len(new_ids)} new, cumulative: {len(all_ids)}")
    print(f"  Cursor: {cursor}")
    print(f"  Pagination: {json.dumps(pagination)}")
    
    if not new_ids or not products:
        print("  No new products, stopping")
        break



#!/usr/bin/env python3
"""Try alternative Webgains feed download endpoints"""
import requests, json

API = "https://platform-api.webgains.com"
PUB = "1347930"
TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
H = {"Authorization": f"Bearer {TOKEN}"}
CAMP = "1622230"
FEED = "17108"  # Van Arendonk NL new - 21850 products

# Try various endpoints
endpoints = [
    # Products endpoint with large size
    (f"{API}/auth/publishers/{PUB}/campaigns/{CAMP}/feeds/products?feeds[]={FEED}&size=5000", "Large size"),
    # Try size=10000
    (f"{API}/auth/publishers/{PUB}/campaigns/{CAMP}/feeds/products?feeds[]={FEED}&size=10000", "Size 10000"),
    # Try CSV export  
    (f"{API}/auth/publishers/{PUB}/campaigns/{CAMP}/feeds/products/export?feeds[]={FEED}", "Export"),
    # Try download
    (f"{API}/auth/publishers/{PUB}/campaigns/{CAMP}/feeds/products/download?feeds[]={FEED}", "Download"),
    # Try feed-specific product endpoint
    (f"{API}/auth/publishers/{PUB}/campaigns/{CAMP}/feeds/{FEED}/products?size=5000", "Feed products"),
]

for url, label in endpoints:
    try:
        r = requests.get(url, headers=H, timeout=30)
        ct = r.headers.get("content-type", "")
        print(f"\n{label}: {r.status_code} ({ct})")
        if r.status_code == 200:
            if "json" in ct:
                data = r.json()
                if isinstance(data, dict):
                    total = data.get("recordsTotal", "?")
                    got = len(data.get("data", []))
                    print(f"  recordsTotal={total}, got={got}")
                    if got > 0:
                        print(f"  First: {data['data'][0].get('title', '')[:60]}")
                        print(f"  Last: {data['data'][-1].get('title', '')[:60]}")
                else:
                    print(f"  Response type: {type(data)}, len: {len(data) if hasattr(data, '__len__') else '?'}")
            else:
                print(f"  Size: {len(r.content)} bytes")
                print(f"  Preview: {r.text[:300]}")
        else:
            print(f"  Body: {r.text[:200]}")
    except Exception as e:
        print(f"{label}: ERROR {e}")

# Also try the old Webgains feed URL format
print("\n=== Trying classic Webgains feed URLs ===")
classic_urls = [
    f"https://feeds.webgains.com/feeds/{FEED}.csv",
    f"https://feeds.webgains.com/{PUB}/{FEED}.csv",
    f"https://productdata.webgains.com/feeds/{FEED}",
]
for url in classic_urls:
    try:
        r = requests.get(url, headers=H, timeout=10, allow_redirects=True)
        print(f"  {url} -> {r.status_code} ({r.headers.get('content-type','')}) size={len(r.content)}")
        if r.status_code == 200:
            print(f"  Preview: {r.text[:300]}")
    except Exception as e:
        print(f"  {url} -> ERROR {e}")



import requests
import re

# Auth
soap_auth = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:ns="http://schemas.tradetracker.com/affiliate">
  <soap:Body>
    <ns:authenticate>
      <ns:customerID>35058</ns:customerID>
      <ns:passphrase>bf0b9eef0da6caf55fe614518375280fe5c4ecca</ns:passphrase>
      <ns:sandbox>false</ns:sandbox>
      <ns:locale>nl_NL</ns:locale>
      <ns:demo>false</ns:demo>
    </ns:authenticate>
  </soap:Body>
</soap:Envelope>'''

r = requests.post('https://ws.tradetracker.com/soap/affiliate',
                  data=soap_auth,
                  headers={'Content-Type': 'text/xml; charset=utf-8', 'SOAPAction': 'authenticate'},
                  timeout=15)
cookies = r.cookies
print(f'Auth: {r.status_code}')

# Get affiliate sites first
soap_sites = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:ns="http://schemas.tradetracker.com/affiliate">
  <soap:Body>
    <ns:getAffiliateSites/>
  </soap:Body>
</soap:Envelope>'''

# The WSDL says getAffiliateSites takes no params. But error says "Missing parameter"
# Maybe we need to set affiliateSiteID on the session first via setAffiliateSiteID
# Or pass it as header. Let me check WSDL...

# Actually, TradeTracker SOAP requires calling getAffiliateSites first, then setAffiliateSite
# But let's try getting WSDL first
print('\n--- Checking WSDL ---')
r_wsdl = requests.get('https://ws.tradetracker.com/soap/affiliate?wsdl', timeout=15)
wsdl = r_wsdl.text

# Find all operations
ops = re.findall(r'<operation name="([^"]+)"', wsdl)
print(f'Operations: {ops}')

# Check if there's a getAffiliateSites that needs params
sites_part = wsdl[wsdl.find('getAffiliateSites'):wsdl.find('getAffiliateSites')+500]
print(f'\ngetAffiliateSites context: {sites_part[:300]}')

# Try with explicit empty params
soap_sites2 = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:ns1="https://ws.tradetracker.com/soap/affiliate">
  <soap:Body>
    <ns1:getAffiliateSites/>
  </soap:Body>
</soap:Envelope>'''

r2 = requests.post('https://ws.tradetracker.com/soap/affiliate',
                   data=soap_sites2,
                   headers={'Content-Type': 'text/xml; charset=utf-8', 'SOAPAction': ''},
                   cookies=cookies, timeout=30)
print(f'\ngetAffiliateSites v2: {r2.status_code}')
print(f'{r2.text[:1000]}')

# Try with the correct namespace from WSDL response
soap_sites3 = '''<?xml version="1.0" encoding="utf-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
                   xmlns:ns1="https://ws.tradetracker.com/soap/affiliate">
  <SOAP-ENV:Body>
    <ns1:getAffiliateSites/>
  </SOAP-ENV:Body>
</SOAP-ENV:Envelope>'''

r3 = requests.post('https://ws.tradetracker.com/soap/affiliate',
                   data=soap_sites3,
                   headers={'Content-Type': 'text/xml; charset=utf-8'},
                   cookies=cookies, timeout=30)
print(f'\ngetAffiliateSites v3: {r3.status_code}')
print(f'{r3.text[:2000]}')



#!/usr/bin/env python3
"""Test JD Sports Awin CSV feed."""
import requests
import gzip
import io
import csv

url = "https://productdata.awin.com/datafeed/download/apikey/489bdc79c6362be4f7d60e6b2506a2de/language/nl/cid/139,614,189,194/fid/77079/rid/0/hasEnhancedFeeds/0/columns/aw_deep_link,product_name,aw_product_id,merchant_product_id,merchant_image_url,description,merchant_category,search_price,merchant_name,merchant_id,category_name,category_id,aw_image_url,currency,store_price,delivery_cost,merchant_deep_link,language,last_updated,display_price,data_feed_id,commission_group,merchant_product_category_path,merchant_product_second_category,merchant_product_third_category,brand_name,brand_id,colour,product_short_description,specifications,condition,product_model,model_number,dimensions,keywords,promotional_text,product_type,merchant_thumb_url,large_image,alternate_image,aw_thumb_url,alternate_image_two,alternate_image_three,alternate_image_four,ean,isbn,upc,mpn,parent_product_id,product_GTIN,basket_link,Fashion%3Asuitable_for,Fashion%3Acategory,Fashion%3Asize,Fashion%3Amaterial,Fashion%3Apattern,Fashion%3Aswatch/format/csv/delimiter/%2C/compression/gzip/adultcontent/1/"

print("Downloading JD Sports feed...")
r = requests.get(url, timeout=120, stream=True)
print(f"Status: {r.status_code}, Content-Type: {r.headers.get('content-type')}")

if r.status_code == 200:
    # Decompress gzip
    data = gzip.decompress(r.content)
    text = data.decode('utf-8', errors='replace')
    reader = csv.DictReader(io.StringIO(text))
    
    rows = []
    for i, row in enumerate(reader):
        rows.append(row)
        if i >= 9:
            break
    
    print(f"\nTotal bytes: {len(data):,}")
    print(f"Columns: {list(rows[0].keys()) if rows else 'none'}")
    print(f"\nFirst 3 products:")
    for row in rows[:3]:
        print(f"  {row.get('brand_name', '?')} - {row.get('product_name', '?')[:80]}")
        print(f"    Price: {row.get('search_price')} {row.get('currency')}")
        print(f"    Category: {row.get('merchant_category', '')[:60]}")
        print(f"    Model: {row.get('product_model', '')}")
        print(f"    Size: {row.get('Fashion:size', '')}")
        print(f"    EAN: {row.get('ean', '')}")
        print(f"    Color: {row.get('colour', '')}")
        print()
    
    # Count total lines
    total = text.count('\n') - 1
    print(f"Total products in feed: ~{total:,}")
    
    # Count sneaker brands
    sneaker_brands = {'nike', 'adidas', 'new balance', 'puma', 'asics', 'jordan', 'reebok', 'converse', 'vans', 'saucony', 'hoka', 'on running', 'salomon'}
    brand_counts = {}
    reader2 = csv.DictReader(io.StringIO(text))
    for row in reader2:
        b = (row.get('brand_name') or '').lower()
        brand_counts[b] = brand_counts.get(b, 0) + 1
    
    print("Top 15 brands:")
    for b, c in sorted(brand_counts.items(), key=lambda x: -x[1])[:15]:
        marker = " *sneaker*" if b in sneaker_brands else ""
        print(f"  {b}: {c}{marker}")



#!/usr/bin/env python3
"""Discover TradeTracker feeds via SOAP with proper affiliate site handling."""
import requests
import re

SOAP_URL = 'https://ws.tradetracker.com/soap/affiliate'
HEADERS = {'Content-Type': 'text/xml; charset=utf-8'}

# Auth
soap_auth = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:ns="http://schemas.tradetracker.com/affiliate">
  <soap:Body>
    <ns:authenticate>
      <ns:customerID>35058</ns:customerID>
      <ns:passphrase>bf0b9eef0da6caf55fe614518375280fe5c4ecca</ns:passphrase>
      <ns:sandbox>false</ns:sandbox>
      <ns:locale>nl_NL</ns:locale>
      <ns:demo>false</ns:demo>
    </ns:authenticate>
  </soap:Body>
</soap:Envelope>'''

r = requests.post(SOAP_URL, data=soap_auth, headers={**HEADERS, 'SOAPAction': 'authenticate'}, timeout=15)
cookies = r.cookies
print(f'Auth: {r.status_code}')

# Get affiliate site IDs - the WSDL shows getAffiliateSitesByFilter might work
# Let's try getAffiliateSites with affiliateSiteFilter
soap_sites = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:ns="http://schemas.tradetracker.com/affiliate">
  <soap:Body>
    <ns:getAffiliateSites>
      <ns:affiliateSiteFilter/>
    </ns:getAffiliateSites>
  </soap:Body>
</soap:Envelope>'''

r2 = requests.post(SOAP_URL, data=soap_sites, headers={**HEADERS, 'SOAPAction': 'getAffiliateSites'}, cookies=cookies, timeout=30)
print(f'\ngetAffiliateSites (with filter): {r2.status_code}')
print(f'Response: {r2.text[:2000]}')

# Extract site IDs
sites = re.findall(r'<item[^>]*>(.*?)</item>', r2.text, re.DOTALL)
print(f'Found {len(sites)} sites')
for s in sites:
    sid = re.search(r'<ID[^>]*>(.*?)</ID>', s)
    sname = re.search(r'<name[^>]*>(.*?)</name>', s)
    if sid:
        print(f'  Site: [{sid.group(1)}] {sname.group(1) if sname else "?"}')

# If we found a site, try getCampaigns with it
if sites:
    site_id = re.search(r'<ID[^>]*>(.*?)</ID>', sites[0]).group(1)
    print(f'\nUsing site ID: {site_id}')
    
    soap_campaigns = f'''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:ns="http://schemas.tradetracker.com/affiliate">
  <soap:Body>
    <ns:getCampaigns>
      <ns:affiliateSiteID>{site_id}</ns:affiliateSiteID>
      <ns:campaignFilter/>
    </ns:getCampaigns>
  </soap:Body>
</soap:Envelope>'''

    r3 = requests.post(SOAP_URL, data=soap_campaigns, headers={**HEADERS, 'SOAPAction': 'getCampaigns'}, cookies=cookies, timeout=30)
    print(f'getCampaigns: {r3.status_code}')
    
    campaigns = re.findall(r'<item[^>]*>(.*?)</item>', r3.text, re.DOTALL)
    print(f'Found {len(campaigns)} campaigns')
    for c in campaigns[:50]:
        cid = re.search(r'<ID[^>]*>(.*?)</ID>', c)
        name = re.search(r'<name[^>]*>(.*?)</name>', c)
        if cid and name:
            cname = name.group(1)
            keywords = ['sneaker', 'schoen', 'shoe', 'sport', 'nike', 'adidas', 'puma', 'asics', 'foot', 'keep', 'mooie']
            is_relevant = any(kw in cname.lower() for kw in keywords)
            marker = ' ***' if is_relevant else ''
            print(f'  [{cid.group(1)}] {cname}{marker}')

    # Try getProductFeeds with site ID
    soap_pf = f'''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:ns="http://schemas.tradetracker.com/affiliate">
  <soap:Body>
    <ns:getProductFeeds>
      <ns:affiliateSiteID>{site_id}</ns:affiliateSiteID>
    </ns:getProductFeeds>
  </soap:Body>
</soap:Envelope>'''

    r4 = requests.post(SOAP_URL, data=soap_pf, headers={**HEADERS, 'SOAPAction': 'getProductFeeds'}, cookies=cookies, timeout=30)
    print(f'\ngetProductFeeds: {r4.status_code}')
    feeds = re.findall(r'<item[^>]*>(.*?)</item>', r4.text, re.DOTALL)
    print(f'Found {len(feeds)} feeds')
    for f in feeds:
        fid = re.search(r'<ID[^>]*>(.*?)</ID>', f)
        name = re.search(r'<name[^>]*>(.*?)</name>', f)
        pcount = re.search(r'<productCount[^>]*>(.*?)</productCount>', f)
        camp_name = re.search(r'<campaignName[^>]*>(.*?)</campaignName>', f)
        url_match = re.search(r'<URL[^>]*>(.*?)</URL>', f)
        if fid and name:
            n = name.group(1)
            cn = camp_name.group(1) if camp_name else '?'
            pc = pcount.group(1) if pcount else '?'
            url_str = url_match.group(1) if url_match else ''
            keywords = ['sneaker', 'schoen', 'shoe', 'sport', 'nike', 'adidas', 'puma', 'asics', 'foot', 'keep', 'mooie']
            is_relevant = any(kw in n.lower() or kw in cn.lower() for kw in keywords)
            marker = ' ***' if is_relevant else ''
            print(f'  [{fid.group(1)}] {n} ({cn}) [{pc} products]{marker}')
            if url_match:
                print(f'    URL: {url_str[:200]}')



#!/usr/bin/env python3
"""Test feedIds pagination and total count."""
import requests, json

TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxMDAwMzEiLCJqdGkiOiIwNjM5ODEwYzQ0YzM1MTBlZmZiMGRkYWNiM2EyNWFlNTg2YzFmNmQ5ODNjNGQ2MDA4NzkwZmM0NTkzY2EyNjg4ZGViZDE4NTgwOThmYjJjYiIsImlhdCI6MTc3MTUxNjc4OS44MzAxMzUsIm5iZiI6MTc3MTUxNjc4OS44MzAxMzcsImV4cCI6MjA4NzA0OTU4OS44MjcwODgsInN1YiI6IjIwMTcwOCIsInNjb3BlcyI6W119.ZxlCDsyBI7_SNUWjO-PUxDYKvQVfuUBzU5n-SgqHdAZCnOj9253yVcJIS5NRXJLrnyx0E_9bn8ymfXS71cvPISA4hPAlKCSMjdkv9f2k2FFwdAlQQ6mdhBB2m9dzHUIn34QwA8s4-ikB1ATKIZeqiJL-3Gh9eiMYPpvaJdCCjN-oGPMhhlehkgA04cZSgwy6HfKgb6uPvUSG9w_tvN8U06uEHKV11DedQHNWWGzuXpd2FEL3baV0z9KjSNMZsJZHx2yHUb6gVWJwaR7_louahMvrPXG-YLgnGvNMcPDPjm-z5HJ-JKcPv9_FBqAYGItZxgeGgVoS8MVhW6lc1a-6CvYtVanfXF42MPPNP-Qq76qRtHPLRBdkIATMuBxVJrMzlprT3n4F0sUUltGR4ixZuzMUm5LeyPMHgmjy2UfYzsl9eVqx0oJCmKUyY9maBXwFLJa8J1NHM_nWp3hPpCGkMyhzHK1DIf_z8Wirg1hp5vKts93eERWJBlQwhucUUS_KnZeC_gVaQ7kT4jCwaEWkpPxPxyXWRoK0mcYS_JGz3JbpVhRLj5DVYIB9k54DXqjYlCx8ezJZE9sf2SX3VWvPvD1xSQD1caTCm0To0B-MkeU1YpEfWdUslpCJXrpOy2Wbs8iDHR1BzNrMnOEr4zbz_EoB4JJe3tGTGjzNLNIw6XI"
PUBLISHER_ID = "1347930"
CAMPAIGN_ID = "1622230"
H = {"Authorization": f"Bearer {TOKEN}", "Accept": "application/json"}
url = f"https://platform-api.webgains.com/auth/publishers/{PUBLISHER_ID}/campaigns/{CAMPAIGN_ID}/feeds/products"

# Test pagination with feedIds[]
print("=== Baskets (18588) - page 1 ===")
params = {"feedIds[]": [18588], "size": 1000, "offset": 0}
r = requests.get(url, headers=H, params=params, timeout=60)
data = r.json()
batch1 = data.get("data", [])
print(f"Got {len(batch1)} products")
print(f"First: {batch1[0].get('brand')} | {batch1[0].get('title','')[:50]}")
print(f"Last:  {batch1[-1].get('brand')} | {batch1[-1].get('title','')[:50]}")
first_ids = set(p.get("id") for p in batch1)

print("\n=== Baskets (18588) - page 2 ===")
params["offset"] = 1000
r2 = requests.get(url, headers=H, params=params, timeout=60)
data2 = r2.json()
batch2 = data2.get("data", [])
print(f"Got {len(batch2)} products")
if batch2:
    print(f"First: {batch2[0].get('brand')} | {batch2[0].get('title','')[:50]}")
    second_ids = set(p.get("id") for p in batch2)
    overlap = first_ids & second_ids
    print(f"Overlap with page 1: {len(overlap)} products")
    if overlap:
        print("⚠️ Pagination broken - returning duplicates")
else:
    print("No more products (total was ~{} from page 1)".format(len(batch1)))

# Quick check: different feeds return different data
print("\n=== Verify feed separation ===")
for fid, name in [(18588, "Baskets"), (16158, "Sneakermood"), (25578, "Patta")]:
    params = {"feedIds[]": [fid], "size": 3}
    r = requests.get(url, headers=H, params=params, timeout=10)
    data = r.json()
    prods = data.get("data", [])
    if prods:
        brands = set(p.get("brand") for p in prods)
        print(f"  Feed {fid} ({name}): {len(prods)} returned, brands={brands}")
        # Check affiliate link domain
        link = prods[0].get("link", "")
        if "baskets" in link.lower():
            print(f"    ✅ Link goes to Baskets")
        elif "patta" in link.lower():
            print(f"    ✅ Link goes to Patta")
        elif "sneakermood" in link.lower():
            print(f"    ✅ Link goes to Sneakermood")
        else:
            print(f"    Link: {link[:100]}")
    else:
        print(f"  Feed {fid} ({name}): NO DATA")
